@@ -7,6 +7,9 @@
 
 set -exu
 
+# Make tracing more educational
+export PS4='+ ${BASH_SOURCE:-}:${FUNCNAME[0]:-}:L${LINENO:-}:   '
+
 LIBVIRT_STORAGE_POOL=${LIBVIRT_STORAGE_POOL:-"default"}
 LIBVIRT_CONNECT_URI=${LIBVIRT_CONNECT_URI:-"qemu:///system"}
 
@@ -16,12 +19,7 @@ export VIRSH_DEFAULT_CONNECT_URI=$LIBVIRT_CONNECT_URI
 
 VOL_NAME="$NAME.qcow2"
 virsh list | grep -q $NAME && virsh destroy $NAME
-virsh list --inactive | grep -q $NAME && virsh undefine $NAME
-
-# Delete the Virtual BMC
-if [[ $(type -P vbmc) != "" ]]; then
-    vbmc list | grep -a $NAME && vbmc delete $NAME
-fi
+virsh list --inactive | grep -q $NAME && virsh undefine $NAME --nvram
 
 if virsh pool-list | grep -q $LIBVIRT_STORAGE_POOL ; then
     virsh vol-list $LIBVIRT_STORAGE_POOL | grep -q $VOL_NAME &&
