@@ -17,13 +17,12 @@ import collections
 
 from oslo_concurrency import lockutils
 from oslo_log import log
-from stevedore import named
+import stevedore
 
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.conf import CONF
 from ironic.drivers import base as driver_base
-from ironic.drivers import fake_hardware
 
 
 LOG = log.getLogger(__name__)
@@ -69,9 +68,7 @@ def _attach_interfaces_to_driver(bare_driver, node, hw_type):
              the requested implementation is not compatible with it.
     """
     for iface in _INTERFACE_LOADERS:
-        iface_name = '%s_interface' % iface
-        impl_name = node.instance_info.get(iface_name,
-                                           getattr(node, iface_name))
+        impl_name = node.get_interface(iface)
         impl = get_interface(hw_type, iface, impl_name)
         setattr(bare_driver, iface, impl)
 
@@ -100,6 +97,7 @@ def get_interface(hw_type, interface_type, interface_name):
             entrypoint=factory._entrypoint_name,
             valid=factory.names)
 
+    from ironic.drivers import fake_hardware  # avoid circular import
     if isinstance(hw_type, fake_hardware.FakeHardware):
         # NOTE(dtantsur): special-case fake hardware type to allow testing with
         # any combinations of interface implementations.
@@ -343,16 +341,31 @@ class BaseDriverFactory(object):
     def get_driver(self, name):
         return self[name].obj
 
-    # NOTE(tenbrae): Use lockutils to avoid a potential race in eventlet
-    #             that might try to create two driver factories.
+    # NOTE(tenbrae): Drivers raise "DriverLoadError" if they are unable to
+    #             be loaded, eg. due to missing external dependencies.
+    #             We capture that exception, and, only if it is for an
+    #             enabled driver, raise it from here. If enabled driver
+    #             raises other exception type, it is wrapped in
+    #             "DriverLoadError", providing the name of the driver that
+    #             caused it, and raised. If the exception is for a
+    #             non-enabled driver, we suppress it.
     @classmethod
-    @lockutils.synchronized(EM_SEMAPHORE)
-    def _init_extension_manager(cls):
-        # NOTE(tenbrae): In case multiple greenthreads queue up on this lock
-        #             before _extension_manager is initialized, prevent
-        #             creation of multiple NameDispatchExtensionManagers.
-        if cls._extension_manager:
-            return
+    def _catch_driver_not_found(cls, mgr, ep, exc):
+        # NOTE(tenbrae): stevedore loads plugins *before* evaluating
+        #             _check_func, so we need to check here, too.
+        if ep.name in cls._enabled_driver_list:
+            if not isinstance(exc, exception.DriverLoadError):
+                raise exception.DriverLoadError(driver=ep.name, reason=exc)
+            raise exc
+
+    @classmethod
+    def _missing_callback(cls, names):
+        names = ', '.join(names)
+        raise exception.DriverNotFoundInEntrypoint(
+            names=names, entrypoint=cls._entrypoint_name)
+
+    @classmethod
+    def _set_enabled_drivers(cls):
         enabled_drivers = getattr(CONF, cls._enabled_driver_list_config_option,
                                   [])
 
@@ -377,39 +390,32 @@ class BaseDriverFactory(object):
                         'configuration file.',
                         ', '.join(duplicated_drivers))
 
-        # NOTE(tenbrae): Drivers raise "DriverLoadError" if they are unable to
-        #             be loaded, eg. due to missing external dependencies.
-        #             We capture that exception, and, only if it is for an
-        #             enabled driver, raise it from here. If enabled driver
-        #             raises other exception type, it is wrapped in
-        #             "DriverLoadError", providing the name of the driver that
-        #             caused it, and raised. If the exception is for a
-        #             non-enabled driver, we suppress it.
-        def _catch_driver_not_found(mgr, ep, exc):
-            # NOTE(tenbrae): stevedore loads plugins *before* evaluating
-            #             _check_func, so we need to check here, too.
-            if ep.name in cls._enabled_driver_list:
-                if not isinstance(exc, exception.DriverLoadError):
-                    raise exception.DriverLoadError(driver=ep.name, reason=exc)
-                raise exc
+    @classmethod
+    def _init_extension_manager(cls):
+        # NOTE(tenbrae): Use lockutils to avoid a potential race
+        # that might try to create two driver factories.
+        with lockutils.lock(cls._entrypoint_name, do_log=False):
+            # NOTE(tenbrae): In case multiple greenthreads queue up on this
+            # lock before _extension_manager is initialized, prevent
+            # creation of multiple NameDispatchExtensionManagers.
+            if cls._extension_manager:
+                return
 
-        def missing_callback(names):
-            names = ', '.join(names)
-            raise exception.DriverNotFoundInEntrypoint(
-                names=names, entrypoint=cls._entrypoint_name)
+            cls._set_enabled_drivers()
 
-        cls._extension_manager = (
-            named.NamedExtensionManager(
-                cls._entrypoint_name,
-                cls._enabled_driver_list,
-                invoke_on_load=True,
-                on_load_failure_callback=_catch_driver_not_found,
-                propagate_map_exceptions=True,
-                on_missing_entrypoints_callback=missing_callback))
+            cls._extension_manager = (
+                stevedore.NamedExtensionManager(
+                    cls._entrypoint_name,
+                    cls._enabled_driver_list,
+                    invoke_on_load=True,
+                    on_load_failure_callback=cls._catch_driver_not_found,
+                    propagate_map_exceptions=True,
+                    on_missing_entrypoints_callback=cls._missing_callback))
 
-        # warn for any untested/unsupported/deprecated drivers or interfaces
-        if cls._enabled_driver_list:
-            cls._extension_manager.map(_warn_if_unsupported)
+            # warn for any untested/unsupported/deprecated drivers or
+            # interfaces
+            if cls._enabled_driver_list:
+                cls._extension_manager.map(_warn_if_unsupported)
 
         LOG.info(cls._logging_template, cls._extension_manager.names())
 
@@ -423,6 +429,34 @@ class BaseDriverFactory(object):
         return ((ext.name, ext.obj) for ext in self._extension_manager)
 
 
+class InterfaceFactory(BaseDriverFactory):
+
+    # Name of a HardwareType attribute with a list of supported interfaces
+    _supported_driver_list_field = ''
+
+    @classmethod
+    def _set_enabled_drivers(cls):
+        super()._set_enabled_drivers()
+        if cls._enabled_driver_list:
+            return
+
+        tmp_ext_mgr = stevedore.ExtensionManager(
+            cls._entrypoint_name,
+            invoke_on_load=False,  # do not create interfaces
+            on_load_failure_callback=cls._catch_driver_not_found)
+        cls_names = {v.plugin: k for (k, v) in tmp_ext_mgr.items()}
+
+        # Fallback: calculate based on hardware type defaults
+        for hw_type in hardware_types().values():
+            supported = getattr(hw_type, cls._supported_driver_list_field)[0]
+            try:
+                name = cls_names[supported]
+            except KeyError:
+                raise KeyError("%s not in %s" % (supported, cls_names))
+            if name not in cls._enabled_driver_list:
+                cls._enabled_driver_list.append(name)
+
+
 def _warn_if_unsupported(ext):
     if not ext.obj.supported:
         LOG.warning('Driver "%s" is UNSUPPORTED. It has been deprecated '
@@ -437,10 +471,12 @@ class HardwareTypesFactory(BaseDriverFactory):
 
 _INTERFACE_LOADERS = {
     name: type('%sInterfaceFactory' % name.capitalize(),
-               (BaseDriverFactory,),
+               (InterfaceFactory,),
                {'_entrypoint_name': 'ironic.hardware.interfaces.%s' % name,
                 '_enabled_driver_list_config_option':
                 'enabled_%s_interfaces' % name,
+                '_supported_driver_list_field':
+                'supported_%s_interfaces' % name,
                 '_logging_template':
                 "Loaded the following %s interfaces: %%s" % name})
     for name in driver_base.ALL_INTERFACES
