@@ -140,6 +140,10 @@ class Invalid(IronicException):
     code = http_client.BAD_REQUEST
 
 
+class MalformedRequestURI(Invalid):
+    _msg_fmt = _("Malformed request URI.")
+
+
 class Conflict(IronicException):
     _msg_fmt = _('Conflict.')
     code = http_client.CONFLICT
@@ -525,10 +529,6 @@ class ServiceUnavailable(IronicException):
     _msg_fmt = _("Connection failed")
 
 
-class Forbidden(IronicException):
-    _msg_fmt = _("Requested OpenStack Baremetal API is forbidden")
-
-
 # TODO(dtantsur): leave only one variant
 BadRequest = Invalid
 
@@ -1100,11 +1100,6 @@ class OciImageNotSpecific(InvalidImage):
                  "utilized for the file download.")
 
 
-class OciImageTagNotFound(ImageNotFound):
-    _msg_fmt = _("Image tag %(image_url)s could not be found, "
-                 "available tags are %(tags)s")
-
-
 class ImageServiceAuthenticationRequired(ImageUnacceptable):
     _msg_fmt = _("The requested image %(image_ref)s requires "
                  "authentication which has not been provided. "
@@ -1140,3 +1135,9 @@ class RuleActionExecutionFailure(InspectionRuleExecutionFailure):
 class RuleConditionCheckFailure(InspectionRuleExecutionFailure):
     """Raised when an inspection rule condition fails during execution."""
     _msg_fmt = _("Inspection rule condition check failed. Reason: %(reason)s")
+
+
+class ConfigDriveRegenerationFailure(IronicException):
+    """Raised when we fail to handle configuration drive corrections."""
+    # NOTE(TheJulia): This is not intended to get raised to a user, but more
+    # so we handle known possible failure cases and don't fail horribly.
