@@ -36,6 +36,11 @@ from ironic.common import utils
 from ironic.conf import CONF
 
 IMAGE_CHUNK_SIZE = 1024 * 1024  # 1mb
+# NOTE(JayF): This is the check-of-last-resort; we also have an allowlist
+# enabled by default. These represent paths that under no circumstances should
+# we access for file:// URLs
+BLOCKED_FILE_URL_PATHS = {'/dev', '/sys', '/proc', '/boot', '/etc', '/run'}
+
 LOG = log.getLogger(__name__)
 
 
@@ -820,6 +825,13 @@ class FileImageService(BaseImageService):
 
         # Check if the path is in the blocklist
         rpath = os.path.abspath(image_path)
+        for bad in BLOCKED_FILE_URL_PATHS:
+            if rpath == bad or rpath.startswith(bad + os.sep):
+                raise exception.ImageRefValidationFailed(
+                    image_href=image_href,
+                    reason=_("Security: The path %s is not permitted in file "
+                             "URLs" % bad)
+                )
 
         # Check if the path is in the allowlist
         for allowed in CONF.conductor.file_url_allowed_paths:
