@@ -20,25 +20,23 @@ https://www.jsonrpc.org/specification. Main differences:
 """
 
 import json
+import logging
 
-from ironic_lib import auth_basic
 from keystonemiddleware import auth_token
 from oslo_config import cfg
-from oslo_log import log
 import oslo_messaging
-from oslo_service import service
-from oslo_service import wsgi
 from oslo_utils import strutils
 import webob
 
-from ironic.common import context as ir_context
+from ironic.common import auth_basic
 from ironic.common import exception
 from ironic.common.i18n import _
-from ironic.common import json_rpc
+from ironic.common import wsgi_service
+from ironic.conf import json_rpc
 
 
 CONF = cfg.CONF
-LOG = log.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 _DENY_LIST = {'init_host', 'del_host', 'target', 'iter_nodes'}
 
 
@@ -84,12 +82,32 @@ class InvalidParams(JsonRpcError):
     _msg_fmt = _("Params %(params)s are invalid for %(method)s: %(error)s")
 
 
-class WSGIService(service.Service):
+class EmptyContext:
+
+    request_id = None
+
+    def __init__(self, src):
+        self.__dict__.update(src)
+
+    def to_dict(self):
+        return self.__dict__.copy()
+
+
+class WSGIService(wsgi_service.BaseWSGIService):
     """Provides ability to launch JSON RPC as a WSGI application."""
 
-    def __init__(self, manager, serializer):
+    def __init__(self, manager, serializer, context_class=EmptyContext):
+        """Create a JSON RPC service.
+
+        :param manager: Object from which to expose methods.
+        :param serializer: A serializer that supports calls serialize_entity
+            and deserialize_entity.
+        :param context_class: A context class - a callable accepting a dict
+            received from network.
+        """
         self.manager = manager
         self.serializer = serializer
+        self.context_class = context_class
         self._method_map = _build_method_map(manager)
         auth_strategy = json_rpc.auth_strategy()
         if auth_strategy == 'keystone':
@@ -101,10 +119,8 @@ class WSGIService(service.Service):
                 cfg.CONF.json_rpc.http_basic_auth_user_file)
         else:
             app = self._application
-        self.server = wsgi.Server(CONF, 'ironic-json-rpc', app,
-                                  host=CONF.json_rpc.host_ip,
-                                  port=CONF.json_rpc.port,
-                                  use_ssl=CONF.json_rpc.use_ssl)
+        super().__init__('ironic-json-rpc', app, CONF.json_rpc)
+        self._debug = CONF.rpc_transport != 'none'
 
     def _application(self, environment, start_response):
         """WSGI application for conductor JSON RPC."""
@@ -117,9 +133,10 @@ class WSGIService(service.Service):
 
         if json_rpc.auth_strategy() == 'keystone':
             roles = (request.headers.get('X-Roles') or '').split(',')
-            if 'admin' not in roles:
-                LOG.debug('Roles %s do not contain "admin", rejecting '
-                          'request', roles)
+            allowed_roles = CONF.json_rpc.allowed_roles
+            if set(roles).isdisjoint(allowed_roles):
+                LOG.debug('Roles %s do not contain any of %s, rejecting '
+                          'request', roles, allowed_roles)
                 body = {'error': {'code': 403, 'message': _('Forbidden')}}
                 return webob.Response(status_code=403, json_body=body)(
                     environment, start_response)
@@ -241,12 +258,14 @@ class WSGIService(service.Service):
                 raise InvalidParams(
                     _("Context must be a dictionary, if provided"))
 
-            context = ir_context.RequestContext.from_dict(context)
+            context = self.context_class(context)
             params = {key: self.serializer.deserialize_entity(context, value)
                       for key, value in params.items()}
             params['context'] = context
 
-        LOG.debug('RPC %s with %s', name, logged_params)
+        if self._debug:
+            LOG.debug('RPC %s with %s', name, logged_params)
+
         try:
             result = func(**params)
         # FIXME(dtantsur): we could use the inspect module, but
@@ -259,35 +278,9 @@ class WSGIService(service.Service):
             # Currently it seems that we can serialize even with invalid
             # context, but I'm not sure it's guaranteed to be the case.
             result = self.serializer.serialize_entity(context, result)
-        LOG.debug('RPC %s returned %s', name,
-                  strutils.mask_dict_password(result)
-                  if isinstance(result, dict) else result)
+
+        if self._debug:
+            LOG.debug('RPC %s returned %s', name,
+                      strutils.mask_dict_password(result)
+                      if isinstance(result, dict) else result)
         return result
-
-    def start(self):
-        """Start serving this service using loaded configuration.
-
-        :returns: None
-        """
-        self.server.start()
-
-    def stop(self):
-        """Stop serving this API.
-
-        :returns: None
-        """
-        self.server.stop()
-
-    def wait(self):
-        """Wait for the service to stop serving this API.
-
-        :returns: None
-        """
-        self.server.wait()
-
-    def reset(self):
-        """Reset server greenpool size to default.
-
-        :returns: None
-        """
-        self.server.reset()
