@@ -14,6 +14,7 @@
 
 import contextlib
 import datetime
+from enum import Enum
 import functools
 import os
 import secrets
@@ -54,6 +55,13 @@ PASSWORD_HASH_FORMAT = {
 }
 
 
+class StepFlow(Enum):
+    CLEANING_MANUAL = 'manual_cleaning'
+    CLEANING_AUTO = 'automated_cleaning'
+    DEPLOYMENT = 'deployment'
+    SERVICING = 'servicing'
+
+
 @task_manager.require_exclusive_lock
 def node_set_boot_device(task, device, persistent=False):
     """Set the boot device for a node.
@@ -637,6 +645,8 @@ def wipe_internal_info_on_power_off(node):
     node.del_driver_internal_info('agent_cached_clean_steps')
     # Remove TLS certificate since it's regenerated on each run.
     node.del_driver_internal_info('agent_verify_ca')
+    # Remove agent start attempt tracking since it doesn't survive power off.
+    node.del_driver_internal_info('agent_start_attempted')
 
 
 def wipe_token_and_url(task):
@@ -663,6 +673,8 @@ def wipe_deploy_internal_info(task):
     node.del_driver_internal_info('deploy_step_index')
     node.del_driver_internal_info('steps_validated')
     node.del_driver_internal_info('image_source')
+    # Remove agent start attempt tracking since deployment is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -676,6 +688,9 @@ def wipe_cleaning_internal_info(task):
     node.del_driver_internal_info('clean_step_index')
     node.del_driver_internal_info('cleaning_disable_ramdisk')
     node.del_driver_internal_info('steps_validated')
+    node.del_driver_internal_info('declarative_cleaning')
+    # Remove agent start attempt tracking since cleaning is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -689,6 +704,8 @@ def wipe_service_internal_info(task):
     node.del_driver_internal_info('service_disable_ramdisk')
     node.del_driver_internal_info('steps_validated')
     node.del_driver_internal_info('image_source')
+    # Remove agent start attempt tracking since servicing is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -1164,6 +1181,7 @@ def power_state_for_network_configuration(task):
     restore_power_state_if_needed(task, previous)
 
 
+# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
 def build_configdrive(node, configdrive):
     """Build a configdrive from provided meta_data, network_data and user_data.
 
@@ -1192,6 +1210,7 @@ def build_configdrive(node, configdrive):
                                 vendor_data=configdrive.get('vendor_data'))
 
 
+# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
 def get_configdrive_image(node):
     """Get configdrive as an ISO image or a URL.
 
@@ -1651,7 +1670,7 @@ def node_cache_boot_mode(task):
         task.node.secure_boot = secure_boot
         task.node.save()
         LOG.info("Updated boot_mode %(boot_mode)s, secure_boot %(secure_boot)s"
-                 "for node %(node)s",
+                 " for node %(node)s",
                  {'boot_mode': boot_mode, 'secure_boot': secure_boot,
                   'node': task.node.uuid})
 
@@ -2005,3 +2024,53 @@ def node_update_cache(task):
     node_cache_boot_mode(task)
     node_cache_bios_settings(task)
     node_cache_firmware_components(task)
+
+
+def log_step_flow_history(node, flow, steps, status, aborted_step=None):
+    """Persist start/end milestones for cleaning / servicing / deploy flows.
+
+    :param node: A Node object
+    :param flow: A StepFlow enum member, e.g. StepFlow.DEPLOYMENT
+    :param steps: full list of step dicts (may be empty/None)
+    :param status: ``'start'`` or ``'done'``
+    :param aborted_step:  name of failed step (optional)
+    """
+    if not (CONF.conductor.record_step_flows_in_history
+            or CONF.conductor.log_step_flows_to_syslog):
+        return
+
+    sanitized = [
+        (
+            dict(st, args=strutils.mask_dict_password(st["args"]))
+            if isinstance(st, dict) and isinstance(st.get("args"), dict)
+            else st
+        )
+        for st in (steps or [])
+    ]
+
+    error = False
+    if status == "start":
+        event_type = "%s_start" % flow.value
+        event = "Starting %s with steps=%s" % (
+            flow.value.replace("_", " "), sanitized)
+    elif status == "end":
+        event_type = "%s_end" % flow.value
+        if aborted_step:
+            error = True
+            event = "%s aborted at step '%s' with steps=%s" % (
+                flow.value.replace("_", " ").title(),
+                aborted_step, sanitized)
+        else:
+            event = "%s completed successfully with steps=%s" % (
+                flow.value.replace("_", " ").title(), sanitized)
+    else:
+        raise ValueError("Unknown status '%s'" % status)
+
+    if CONF.conductor.log_step_flows_to_syslog:
+        log_fn = LOG.warning if error else LOG.info
+        log_fn("Node %(node)s: %(event)s",
+               {'node': node.uuid, 'event': event})
+
+    if CONF.conductor.record_step_flows_in_history:
+        node_history_record(node, event=event, event_type=event_type,
+                            error=error)
