@@ -23,13 +23,13 @@ from ironic.drivers.modules.ansible import deploy as ansible_deploy
 from ironic.drivers.modules import fake
 from ironic.drivers.modules import inspector
 from ironic.drivers.modules import ipxe
-from ironic.drivers.modules import iscsi_deploy
 from ironic.drivers.modules.network import flat as flat_net
 from ironic.drivers.modules.network import neutron
 from ironic.drivers.modules.network import noop as noop_net
 from ironic.drivers.modules import noop
 from ironic.drivers.modules import noop_mgmt
 from ironic.drivers.modules import pxe
+from ironic.drivers.modules import ramdisk
 from ironic.drivers.modules.storage import cinder
 from ironic.drivers.modules.storage import external as external_storage
 from ironic.drivers.modules.storage import noop as noop_storage
@@ -44,20 +44,21 @@ class GenericHardware(hardware_type.AbstractHardwareType):
     @property
     def supported_boot_interfaces(self):
         """List of supported boot interfaces."""
-        return [ipxe.iPXEBoot, pxe.PXEBoot]
+        return [ipxe.iPXEBoot, pxe.PXEBoot, ipxe.iPXEHttpBoot, pxe.HttpBoot]
 
     @property
     def supported_deploy_interfaces(self):
         """List of supported deploy interfaces."""
-        return [agent.AgentDeploy, iscsi_deploy.ISCSIDeploy,
-                ansible_deploy.AnsibleDeploy, pxe.PXERamdiskDeploy]
+        return [agent.AgentDeploy, ansible_deploy.AnsibleDeploy,
+                ramdisk.RamdiskDeploy, pxe.PXEAnacondaDeploy,
+                agent.BootcAgentDeploy, agent.CustomAgentDeploy]
 
     @property
     def supported_inspect_interfaces(self):
         """List of supported inspect interfaces."""
         # Inspector support should be the default if it's enabled by an
         # operator (implying that the service is installed).
-        return [inspector.Inspector, noop.NoInspect]
+        return [inspector.Inspector, inspector.AgentInspect, noop.NoInspect]
 
     @property
     def supported_network_interfaces(self):
@@ -85,6 +86,11 @@ class GenericHardware(hardware_type.AbstractHardwareType):
         return [noop_storage.NoopStorage, cinder.CinderStorage,
                 external_storage.ExternalStorage]
 
+    @property
+    def supported_firmware_interfaces(self):
+        """List of supported firmware interfaces."""
+        return [noop.NoFirmware]
+
 
 class ManualManagementHardware(GenericHardware):
     """Hardware type that uses manual power and boot management.
