@@ -15,12 +15,10 @@
 Boot Interface for iLO drivers and its supporting methods.
 """
 
-from urllib import parse as urlparse
-
-from ironic_lib import metrics_utils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import strutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -28,6 +26,7 @@ from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import images
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.conductor import utils as manager_utils
 from ironic.drivers import base
@@ -37,6 +36,7 @@ from ironic.drivers.modules.ilo import common as ilo_common
 from ironic.drivers.modules import image_utils
 from ironic.drivers.modules import ipxe
 from ironic.drivers.modules import pxe
+from ironic.drivers import utils as driver_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -45,41 +45,40 @@ METRICS = metrics_utils.get_metrics_logger(__name__)
 CONF = cfg.CONF
 
 REQUIRED_PROPERTIES = {
-    'ilo_deploy_iso': _("UUID (from Glance) of the deployment ISO. "
-                        "Required.")
+    'deploy_iso': _("UUID (from Glance) of the deployment ISO. Required.")
 }
 RESCUE_PROPERTIES = {
-    'ilo_rescue_iso': _("UUID (from Glance) of the rescue ISO. Only "
-                        "required if rescue mode is being used and ironic is "
-                        "managing booting the rescue ramdisk.")
+    'rescue_iso': _("UUID (from Glance) of the rescue ISO. Only "
+                    "required if rescue mode is being used and ironic is "
+                    "managing booting the rescue ramdisk.")
 }
 REQUIRED_PROPERTIES_UEFI_HTTPS_BOOT = {
-    'ilo_deploy_kernel': _("URL or Glance UUID of the deployment kernel. "
-                           "Required."),
-    'ilo_deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is "
-                            "mounted at boot time. Required."),
+    'deploy_kernel': _("URL or Glance UUID of the deployment kernel. "
+                       "Required."),
+    'deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is "
+                        "mounted at boot time. Required."),
 }
 RESCUE_PROPERTIES_UEFI_HTTPS_BOOT = {
-    'ilo_rescue_kernel': _('URL or Glance UUID of the rescue kernel. This '
-                           'value is required for rescue mode.'),
-    'ilo_rescue_ramdisk': _('URL or Glance UUID of the rescue ramdisk with '
-                            'agent that is used at node rescue time. '
-                            'The value is required for rescue mode.'),
+    'rescue_kernel': _('URL or Glance UUID of the rescue kernel. This '
+                       'value is required for rescue mode.'),
+    'rescue_ramdisk': _('URL or Glance UUID of the rescue ramdisk with '
+                        'agent that is used at node rescue time. '
+                        'The value is required for rescue mode.'),
 }
 OPTIONAL_PROPERTIES = {
-    'ilo_bootloader': _("URL or Glance UUID  of the EFI system partition "
-                        "image containing EFI boot loader. This image will "
-                        "be used by ironic when building UEFI-bootable ISO "
-                        "out of kernel and ramdisk. Required for UEFI "
-                        "boot from partition images.")
+    'bootloader': _("URL or Glance UUID  of the EFI system partition "
+                    "image containing EFI boot loader. This image will "
+                    "be used by ironic when building UEFI-bootable ISO "
+                    "out of kernel and ramdisk. Required for UEFI "
+                    "boot from partition images."),
+    'ilo_add_certificates': _("Boolean value that indicates whether the "
+                              "certificates require to be added to the "
+                              "iLO."),
+    'kernel_append_params': driver_utils.KERNEL_APPEND_PARAMS_DESCRIPTION %
+    {'option_group': 'ilo'},
 }
 COMMON_PROPERTIES = REQUIRED_PROPERTIES
 
-KERNEL_RAMDISK_LABELS = {
-    'deploy': REQUIRED_PROPERTIES_UEFI_HTTPS_BOOT,
-    'rescue': RESCUE_PROPERTIES_UEFI_HTTPS_BOOT
-}
-
 
 def parse_driver_info(node, mode='deploy'):
     """Gets the driver specific Node deployment info.
@@ -97,32 +96,22 @@ def parse_driver_info(node, mode='deploy'):
     :raises: MissingParameterValue, if any of the required parameters are
         missing.
     """
-    info = node.driver_info
     d_info = {}
-    if mode == 'rescue' and info.get('ilo_rescue_iso'):
-        d_info['ilo_rescue_iso'] = info.get('ilo_rescue_iso')
-    elif mode == 'deploy' and info.get('ilo_deploy_iso'):
-        d_info['ilo_deploy_iso'] = info.get('ilo_deploy_iso')
+    iso_ref = driver_utils.get_agent_iso(node, mode, deprecated_prefix='ilo')
+    if iso_ref:
+        d_info[f'{mode}_iso'] = iso_ref
     else:
-        params_to_check = KERNEL_RAMDISK_LABELS[mode]
+        d_info = driver_utils.get_agent_kernel_ramdisk(
+            node, mode, deprecated_prefix='ilo')
+        d_info['bootloader'] = driver_utils.get_field(node, 'bootloader',
+                                                      deprecated_prefix='ilo',
+                                                      use_conf=True)
 
-        d_info = {option: info.get(option)
-                  for option in params_to_check}
-
-        if not any(d_info.values()):
-            # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
-            # from driver_info but deploy_ramdisk comes from configuration,
-            # since it's a sign of a potential operator's mistake.
-            d_info = {k: getattr(CONF.conductor, k.replace('ilo_', ''))
-                      for k in params_to_check}
-
-    error_msg = (_("Error validating iLO virtual media for %s. Some "
-                   "parameters were missing in node's driver_info.") % mode)
+    error_msg = (_("Error validating iLO boot for %s. Some "
+                   "parameters were missing in node's driver_info") % mode)
     deploy_utils.check_for_missing_params(d_info, error_msg)
-
-    d_info.update(
-        {k: info.get(k, getattr(CONF.conductor, k.replace('ilo_', ''), None))
-         for k in OPTIONAL_PROPERTIES})
+    d_info['kernel_append_params'] = node.driver_info.get(
+        'kernel_append_params')
 
     return d_info
 
@@ -131,7 +120,7 @@ def _get_boot_iso(task, root_uuid):
     """This method returns a boot ISO to boot the node.
 
     It chooses one of the three options in the order as below:
-    1. Does nothing if 'ilo_boot_iso' is present in node's instance_info.
+    1. Does nothing if 'boot_iso' is present in node's instance_info.
     2. Image deployed has a meta-property 'boot_iso' in Glance. This should
        refer to the UUID of the boot_iso which exists in Glance.
     3. Returns a boot ISO created on the fly using kernel and ramdisk
@@ -139,57 +128,41 @@ def _get_boot_iso(task, root_uuid):
 
     :param task: a TaskManager instance containing the node to act on.
     :param root_uuid: the uuid of the root partition.
-    :returns: boot ISO URL. Should be either of below:
-        * A Swift object - It should be of format 'swift:<object-name>'. It is
-          assumed that the image object is present in
-          CONF.ilo.swift_ilo_container;
-        * A Glance image - It should be format 'glance://<glance-image-uuid>'
-          or just <glance-image-uuid>;
-        * An HTTP URL.
-        On error finding the boot iso, it returns None.
+    :returns: boot ISO HTTP or swift:// URL.
     :raises: MissingParameterValue, if any of the required parameters are
         missing in the node's driver_info or instance_info.
     :raises: InvalidParameterValue, if any of the parameters have invalid
         value in the node's driver_info or instance_info.
     :raises: SwiftOperationError, if operation with Swift fails.
     :raises: ImageCreationFailed, if creation of boot ISO failed.
-    :raises: exception.ImageRefValidationFailed if ilo_boot_iso is not
+    :raises: exception.ImageRefValidationFailed if boot_iso is not
         HTTP(S) URL.
     """
     LOG.debug("Trying to get a boot ISO to boot the baremetal node")
 
-    # Option 1 - Check if user has provided ilo_boot_iso in node's
+    # Option 1 - Check if user has provided boot_iso in node's
     # instance_info
-    if task.node.instance_info.get('ilo_boot_iso'):
-        LOG.debug("Using ilo_boot_iso provided in node's instance_info")
-        boot_iso = task.node.instance_info['ilo_boot_iso']
-        if not service_utils.is_glance_image(boot_iso):
-            try:
-                image_service.HttpImageService().validate_href(boot_iso)
-            except exception.ImageRefValidationFailed:
-                with excutils.save_and_reraise_exception():
-                    LOG.error("Virtual media deploy accepts only Glance "
-                              "images or HTTP(S) URLs as "
-                              "instance_info['ilo_boot_iso']. Either %s "
-                              "is not a valid HTTP(S) URL or is "
-                              "not reachable.", boot_iso)
-
-        return task.node.instance_info['ilo_boot_iso']
+    boot_iso = driver_utils.get_field(task.node, 'boot_iso',
+                                      deprecated_prefix='ilo',
+                                      collection='instance_info')
+    deploy_info = _parse_deploy_info(task.node)
 
     # Option 2 - Check if user has provided a boot_iso in Glance. If boot_iso
     # is a supported non-glance href execution will proceed to option 3.
-    deploy_info = _parse_deploy_info(task.node)
+    if not boot_iso:
+        # TODO(dtantsur): this approach should be generic across drivers.
+        image_href = deploy_info['image_source']
+        image_properties = (
+            images.get_image_properties(
+                task.context, image_href, ['boot_iso']))
 
-    image_href = deploy_info['image_source']
-    image_properties = (
-        images.get_image_properties(
-            task.context, image_href, ['boot_iso']))
+        boot_iso = image_properties.get('boot_iso')
 
-    boot_iso_uuid = image_properties.get('boot_iso')
-
-    if boot_iso_uuid:
-        LOG.debug("Found boot_iso %s in Glance", boot_iso_uuid)
-        return boot_iso_uuid
+    if boot_iso:
+        i_info = task.node.instance_info
+        i_info['boot_iso'] = boot_iso
+        task.node.instance_info = i_info
+        task.node.save()
 
     # NOTE(rameshg87): Functionality to share the boot ISOs created for
     # similar instances (instances with same deployed image) is
@@ -251,14 +224,7 @@ def _validate_instance_image_info(task):
     node = task.node
 
     d_info = _parse_deploy_info(node)
-
-    if node.driver_internal_info.get('is_whole_disk_image'):
-        props = []
-    elif service_utils.is_glance_image(d_info['image_source']):
-        props = ['kernel_id', 'ramdisk_id']
-    else:
-        props = ['kernel', 'ramdisk']
-    deploy_utils.validate_image_properties(task.context, d_info, props)
+    deploy_utils.validate_image_properties(task, d_info)
 
 
 def _disable_secure_boot(task):
@@ -318,30 +284,10 @@ def prepare_node_for_deploy(task):
         # not provided.
         # Since secure boot was disabled, we are in 'uefi' boot mode.
         if boot_mode_utils.get_boot_mode_for_deploy(task.node) is None:
-            instance_info = task.node.instance_info
-            instance_info['deploy_boot_mode'] = 'uefi'
-            task.node.instance_info = instance_info
+            task.node.set_driver_internal_info('deploy_boot_mode', 'uefi')
             task.node.save()
 
 
-def disable_secure_boot_if_supported(task):
-    """Disables secure boot on node, does not throw if its not supported.
-
-    :param task: a TaskManager instance containing the node to act on.
-    :raises: IloOperationError, if some operation on iLO failed.
-    """
-    try:
-        ilo_common.update_secure_boot_mode(task, False)
-    # We need to handle IloOperationNotSupported exception so that if
-    # the user has incorrectly specified the Node capability
-    # 'secure_boot' to a node that does not have that capability and
-    # attempted deploy. Handling this exception here, will help the
-    # user to tear down such a Node.
-    except exception.IloOperationNotSupported:
-        LOG.warning('Secure boot mode is not supported for node %s',
-                    task.node.uuid)
-
-
 class IloVirtualMediaBoot(base.BootInterface):
 
     capabilities = ['iscsi_volume_boot', 'ramdisk_boot']
@@ -350,7 +296,7 @@ class IloVirtualMediaBoot(base.BootInterface):
         # TODO(stendulker): COMMON_PROPERTIES should also include rescue
         # related properties (RESCUE_PROPERTIES). We can add them in Rocky,
         # when classic drivers get removed.
-        return COMMON_PROPERTIES
+        return dict(driver_utils.OPTIONAL_PROPERTIES, **COMMON_PROPERTIES)
 
     @METRICS.timer('IloVirtualMediaBoot.validate')
     def validate(self, task):
@@ -364,17 +310,19 @@ class IloVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
         boot_option = deploy_utils.get_boot_option(node)
-        boot_iso = node.instance_info.get('ilo_boot_iso')
-        if (boot_option == "ramdisk" and boot_iso):
+        boot_iso = driver_utils.get_field(node, 'boot_iso',
+                                          deprecated_prefix='ilo',
+                                          collection='instance_info')
+        if boot_option == "ramdisk" and boot_iso:
             if not service_utils.is_glance_image(boot_iso):
                 try:
                     image_service.HttpImageService().validate_href(boot_iso)
                 except exception.ImageRefValidationFailed:
                     with excutils.save_and_reraise_exception():
                         LOG.error("Virtual media deploy with 'ramdisk' "
-                                  "boot_option accepts only Glance images or "
+                                  "deploy accepts only Glance images or "
                                   "HTTP(S) URLs as "
-                                  "instance_info['ilo_boot_iso']. Either %s "
+                                  "instance_info['boot_iso']. Either %s "
                                   "is not a valid HTTP(S) URL or is not "
                                   "reachable.", boot_iso)
             return
@@ -394,12 +342,7 @@ class IloVirtualMediaBoot(base.BootInterface):
             parameters
         :raises: UnsupportedDriverExtension
         """
-        try:
-            _validate_driver_info(task)
-        except exception.MissingParameterValue:
-            # Fall back to non-managed in-band inspection
-            raise exception.UnsupportedDriverExtension(
-                driver=task.node.driver, extension='inspection')
+        _validate_driver_info(task)
 
     @METRICS.timer('IloVirtualMediaBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
@@ -422,24 +365,18 @@ class IloVirtualMediaBoot(base.BootInterface):
         """
 
         node = task.node
-        # NOTE(TheJulia): If this method is being called by something
-        # aside from deployment, clean and rescue, such as conductor takeover,
-        # we should treat this as a no-op and move on otherwise we would
-        # modify the state of the node due to virtual media operations.
-        if node.provision_state not in (states.DEPLOYING,
-                                        states.CLEANING,
-                                        states.RESCUING,
-                                        states.INSPECTING):
+        if not driver_utils.need_prepare_ramdisk(node):
             return
 
         prepare_node_for_deploy(task)
 
-        # Clear ilo_boot_iso if it's a glance image to force recreate
+        # Clear boot_iso if it's a glance image to force recreate
         # another one again (or use existing one in glance).
         # This is mainly for rebuild and rescue scenario.
         if service_utils.is_glance_image(
                 node.instance_info.get('image_source')):
             instance_info = node.instance_info
+            instance_info.pop('boot_iso', None)
             instance_info.pop('ilo_boot_iso', None)
             node.instance_info = instance_info
             node.save()
@@ -455,22 +392,20 @@ class IloVirtualMediaBoot(base.BootInterface):
             task.node.driver_internal_info['agent_secret_token']
         task.node.save()
 
+        ramdisk_params['boot_method'] = 'vmedia'
+
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
-        if (node.driver_info.get('ilo_rescue_iso')
-                and node.provision_state == states.RESCUING):
-            iso = node.driver_info['ilo_rescue_iso']
-            ilo_common.setup_vmedia(task, iso, ramdisk_params)
-        elif node.driver_info.get('ilo_deploy_iso'):
-            iso = node.driver_info['ilo_deploy_iso']
-            ilo_common.setup_vmedia(task, iso, ramdisk_params)
-        else:
-            mode = deploy_utils.rescue_or_deploy_mode(node)
-            d_info = parse_driver_info(node, mode)
-            iso = image_utils.prepare_deploy_iso(task, ramdisk_params,
-                                                 mode, d_info)
-            ilo_common.setup_vmedia(task, iso)
+
+        mode = deploy_utils.rescue_or_deploy_mode(node)
+        d_info = parse_driver_info(node, mode)
+        iso = image_utils.prepare_deploy_iso(task, ramdisk_params,
+                                             mode, d_info)
+        if not d_info.get(f'{mode}_iso'):
+            # No need to attach a floopy when an ISO is built by us.
+            ramdisk_params = None
+        ilo_common.setup_vmedia(task, iso, ramdisk_params)
 
     @METRICS.timer('IloVirtualMediaBoot.prepare_instance')
     def prepare_instance(self, task):
@@ -505,9 +440,7 @@ class IloVirtualMediaBoot(base.BootInterface):
             # It will set iSCSI info onto iLO
             if boot_mode == 'uefi':
                 # Need to set 'ilo_uefi_iscsi_boot' param for clean up
-                driver_internal_info = task.node.driver_internal_info
-                driver_internal_info['ilo_uefi_iscsi_boot'] = True
-                task.node.driver_internal_info = driver_internal_info
+                task.node.set_driver_internal_info('ilo_uefi_iscsi_boot', True)
                 task.node.save()
                 task.driver.management.set_iscsi_boot_target(task)
                 manager_utils.node_set_boot_device(
@@ -522,25 +455,12 @@ class IloVirtualMediaBoot(base.BootInterface):
                                                boot_devices.CDROM,
                                                persistent=True)
         else:
-            # Boot from disk every time if the image deployed is
-            # a whole disk image.
-            node = task.node
-            iwdi = node.driver_internal_info.get('is_whole_disk_image')
-            if deploy_utils.get_boot_option(node) == "local" or iwdi:
-                manager_utils.node_set_boot_device(task, boot_devices.DISK,
-                                                   persistent=True)
-            else:
-                drv_int_info = node.driver_internal_info
-                root_uuid_or_disk_id = drv_int_info.get('root_uuid_or_disk_id')
-                if root_uuid_or_disk_id:
-                    self._configure_vmedia_boot(task, root_uuid_or_disk_id)
-                else:
-                    LOG.warning("The UUID for the root partition could not "
-                                "be found for node %s", node.uuid)
+            manager_utils.node_set_boot_device(task, boot_devices.DISK,
+                                               persistent=True)
         # Set boot mode
         ilo_common.update_boot_mode(task)
         # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
 
     @METRICS.timer('IloVirtualMediaBoot.clean_up_instance')
     def clean_up_instance(self, task):
@@ -557,15 +477,13 @@ class IloVirtualMediaBoot(base.BootInterface):
         """
         LOG.debug("Cleaning up the instance.")
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
         if (deploy_utils.is_iscsi_boot(task)
             and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
             # It will clear iSCSI info from iLO
             task.driver.management.clear_iscsi_boot_target(task)
-            driver_internal_info = task.node.driver_internal_info
-            driver_internal_info.pop('ilo_uefi_iscsi_boot', None)
-            task.node.driver_internal_info = driver_internal_info
+            task.node.del_driver_internal_info('ilo_uefi_iscsi_boot')
             task.node.save()
         else:
             image_utils.cleanup_iso_image(task)
@@ -583,12 +501,7 @@ class IloVirtualMediaBoot(base.BootInterface):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         ilo_common.cleanup_vmedia_boot(task)
-
-        info = task.node.driver_info
-        mode = deploy_utils.rescue_or_deploy_mode(task.node)
-        if ((mode == 'rescue' and not info.get('ilo_rescue_iso'))
-                or (mode == 'deploy' and not info.get('ilo_deploy_iso'))):
-            image_utils.cleanup_iso_image(task)
+        image_utils.cleanup_iso_image(task)
 
     def _configure_vmedia_boot(self, task, root_uuid):
         """Configure vmedia boot for the node.
@@ -601,9 +514,6 @@ class IloVirtualMediaBoot(base.BootInterface):
 
         node = task.node
         boot_iso = _get_boot_iso(task, root_uuid)
-        if not boot_iso:
-            LOG.error("Cannot get boot ISO for node %s", node.uuid)
-            return
 
         # Upon deploy complete, some distros cloud images reboot the system as
         # part of its configuration. Hence boot device should be persistent and
@@ -614,7 +524,7 @@ class IloVirtualMediaBoot(base.BootInterface):
                                            persistent=True)
 
         i_info = node.instance_info
-        i_info['ilo_boot_iso'] = boot_iso
+        i_info['boot_iso'] = boot_iso
         node.instance_info = i_info
         node.save()
 
@@ -662,8 +572,7 @@ class IloPXEBoot(pxe.PXEBoot):
         """Prepares the boot of instance.
 
         This method prepares the boot of the instance after reading
-        relevant information from the node's instance_info. In case of netboot,
-        it updates the dhcp entries and switches the PXE config. In case of
+        relevant information from the node's instance_info. In case of
         localboot, it cleans up the PXE config.
         In case of 'boot from volume', it updates the iSCSI info onto iLO and
         sets the node to boot from 'UefiTarget' boot device.
@@ -672,19 +581,16 @@ class IloPXEBoot(pxe.PXEBoot):
         :returns: None
         :raises: IloOperationError, if some operation on iLO failed.
         """
-
         # Set boot mode
         ilo_common.update_boot_mode(task)
-        # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
 
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
         if deploy_utils.is_iscsi_boot(task) and boot_mode == 'uefi':
+            # Need to enable secure boot, if being requested
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             # Need to set 'ilo_uefi_iscsi_boot' param for clean up
-            driver_internal_info = task.node.driver_internal_info
-            driver_internal_info['ilo_uefi_iscsi_boot'] = True
-            task.node.driver_internal_info = driver_internal_info
+            task.node.set_driver_internal_info('ilo_uefi_iscsi_boot', True)
             task.node.save()
             # It will set iSCSI info onto iLO
             task.driver.management.set_iscsi_boot_target(task)
@@ -693,6 +599,12 @@ class IloPXEBoot(pxe.PXEBoot):
         else:
             # Volume boot in BIOS boot mode is handled using
             # PXE boot interface
+            boot_option = deploy_utils.get_boot_option(task.node)
+            if boot_option == "kickstart":
+                if task.node.provision_state in (states.DEPLOYING,
+                                                 states.RESCUING,
+                                                 states.CLEANING):
+                    prepare_node_for_deploy(task)
             super(IloPXEBoot, self).prepare_instance(task)
 
     @METRICS.timer('IloPXEBoot.clean_up_instance')
@@ -710,16 +622,14 @@ class IloPXEBoot(pxe.PXEBoot):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
-        driver_internal_info = task.node.driver_internal_info
 
         if (deploy_utils.is_iscsi_boot(task)
-            and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+                and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+            boot_mode_utils.deconfigure_secure_boot_if_needed(task)
             # It will clear iSCSI info from iLO in case of booting from
             # volume in UEFI boot mode
             task.driver.management.clear_iscsi_boot_target(task)
-            driver_internal_info.pop('ilo_uefi_iscsi_boot', None)
-            task.node.driver_internal_info = driver_internal_info
+            task.node.del_driver_internal_info('ilo_uefi_iscsi_boot')
             task.node.save()
         else:
             # Volume boot in BIOS boot mode is handled using
@@ -760,8 +670,7 @@ class IloiPXEBoot(ipxe.iPXEBoot):
         """Prepares the boot of instance.
 
         This method prepares the boot of the instance after reading
-        relevant information from the node's instance_info. In case of netboot,
-        it updates the dhcp entries and switches the PXE config. In case of
+        relevant information from the node's instance_info. In case of
         localboot, it cleans up the PXE config.
         In case of 'boot from volume', it updates the iSCSI info onto iLO and
         sets the node to boot from 'UefiTarget' boot device.
@@ -773,16 +682,13 @@ class IloiPXEBoot(ipxe.iPXEBoot):
 
         # Set boot mode
         ilo_common.update_boot_mode(task)
-        # Need to enable secure boot, if being requested
-        ilo_common.update_secure_boot_mode(task, True)
-
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
         if deploy_utils.is_iscsi_boot(task) and boot_mode == 'uefi':
+            # Need to enable secure boot, if being requested
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             # Need to set 'ilo_uefi_iscsi_boot' param for clean up
-            driver_internal_info = task.node.driver_internal_info
-            driver_internal_info['ilo_uefi_iscsi_boot'] = True
-            task.node.driver_internal_info = driver_internal_info
+            task.node.set_driver_internal_info('ilo_uefi_iscsi_boot', True)
             task.node.save()
             # It will set iSCSI info onto iLO
             task.driver.management.set_iscsi_boot_target(task)
@@ -791,6 +697,12 @@ class IloiPXEBoot(ipxe.iPXEBoot):
         else:
             # Volume boot in BIOS boot mode is handled using
             # PXE boot interface
+            boot_option = deploy_utils.get_boot_option(task.node)
+            if boot_option == "kickstart":
+                if task.node.provision_state in (states.DEPLOYING,
+                                                 states.RESCUING,
+                                                 states.CLEANING):
+                    prepare_node_for_deploy(task)
             super(IloiPXEBoot, self).prepare_instance(task)
 
     @METRICS.timer('IloiPXEBoot.clean_up_instance')
@@ -808,16 +720,14 @@ class IloiPXEBoot(ipxe.iPXEBoot):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
-        disable_secure_boot_if_supported(task)
-        driver_internal_info = task.node.driver_internal_info
 
         if (deploy_utils.is_iscsi_boot(task)
-            and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+                and task.node.driver_internal_info.get('ilo_uefi_iscsi_boot')):
+            boot_mode_utils.deconfigure_secure_boot_if_needed(task)
             # It will clear iSCSI info from iLO in case of booting from
             # volume in UEFI boot mode
             task.driver.management.clear_iscsi_boot_target(task)
-            driver_internal_info.pop('ilo_uefi_iscsi_boot', None)
-            task.node.driver_internal_info = driver_internal_info
+            task.node.del_driver_internal_info('ilo_uefi_iscsi_boot')
             task.node.save()
         else:
             # Volume boot in BIOS boot mode is handled using
@@ -850,10 +760,8 @@ class IloUefiHttpsBoot(base.BootInterface):
 
         for prop in image_dict:
             image_ref = image_dict.get(prop)
-            if not service_utils.is_glance_image(image_ref):
-                prefix = urlparse.urlparse(image_ref).scheme.lower()
-                if prefix == 'http':
-                    insecure_props.append(prop)
+            if image_ref is not None and image_ref.startswith('http://'):
+                insecure_props.append(image_ref)
 
         if len(insecure_props) > 0:
             error = (_('Secure URLs exposed over HTTPS are expected. '
@@ -895,35 +803,27 @@ class IloUefiHttpsBoot(base.BootInterface):
         :returns: A dict with the driver_info values.
         :raises: MissingParameterValue, if any of the required parameters are
             missing.
+        :raises: InvalidParameterValue, if any of the required parameters are
+            invalid.
         """
-        info = node.driver_info
+        deploy_info = parse_driver_info(node, mode)
 
-        if mode == 'rescue':
-            params_to_check = RESCUE_PROPERTIES_UEFI_HTTPS_BOOT.keys()
-        else:
-            params_to_check = REQUIRED_PROPERTIES_UEFI_HTTPS_BOOT.keys()
+        should_add_certs = node.driver_info.get('ilo_add_certificates', True)
 
-        deploy_info = {option: info.get(option)
-                       for option in params_to_check}
-
-        if not any(deploy_info.values()):
-            # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
-            # from driver_info but deploy_ramdisk comes from configuration,
-            # since it's a sign of a potential operator's mistake.
-            deploy_info = {k: getattr(CONF.conductor, k.replace('ilo_', ''))
-                           for k in params_to_check}
-
-        deploy_info.update(
-            {k: info.get(k, getattr(CONF.conductor,
-                                    k.replace('ilo_', ''), None))
-             for k in OPTIONAL_PROPERTIES})
+        if should_add_certs is not None:
+            try:
+                should_add_certs = strutils.bool_from_string(should_add_certs,
+                                                             strict=True)
+            except ValueError:
+                raise exception.InvalidParameterValue(
+                    _('Invalid value type set in driver_info/'
+                      'ilo_add_certificates on node %(node)s. '
+                      'The value should be a Boolean '
+                      ' not "%(value)s"'
+                      ) % {'value': should_add_certs, 'node': node.uuid})
 
         self._validate_hrefs(deploy_info)
 
-        error_msg = (_("Error validating %s for iLO UEFI HTTPS boot. Some "
-                       "parameters were missing in node's driver_info") % mode)
-        deploy_utils.check_for_missing_params(deploy_info, error_msg)
-
         deploy_info.update(ilo_common.parse_driver_info(node))
 
         return deploy_info
@@ -957,14 +857,7 @@ class IloUefiHttpsBoot(base.BootInterface):
         d_info = deploy_utils.get_image_instance_info(node)
 
         self._validate_hrefs(d_info)
-
-        if node.driver_internal_info.get('is_whole_disk_image'):
-            props = []
-        elif service_utils.is_glance_image(d_info['image_source']):
-            props = ['kernel_id', 'ramdisk_id']
-        else:
-            props = ['kernel', 'ramdisk']
-        deploy_utils.validate_image_properties(task.context, d_info, props)
+        deploy_utils.validate_image_properties(task, d_info)
 
     @METRICS.timer('IloUefiHttpsBoot.validate')
     def validate(self, task):
@@ -993,17 +886,19 @@ class IloUefiHttpsBoot(base.BootInterface):
                       "The node is required to be in 'UEFI' boot mode.")
             raise exception.InvalidParameterValue(error)
 
-        boot_iso = node.instance_info.get('ilo_boot_iso')
-        if (boot_option == "ramdisk" and boot_iso):
+        boot_iso = driver_utils.get_field(node, 'boot_iso',
+                                          deprecated_prefix='ilo',
+                                          collection='instance_info')
+        if boot_option == "ramdisk" and boot_iso:
             if not service_utils.is_glance_image(boot_iso):
                 try:
                     image_service.HttpImageService().validate_href(boot_iso)
                 except exception.ImageRefValidationFailed:
                     with excutils.save_and_reraise_exception():
                         LOG.error("UEFI-HTTPS boot with 'ramdisk' "
-                                  "boot_option accepts only Glance images or "
+                                  "deploy accepts only Glance images or "
                                   "HTTPS URLs as "
-                                  "instance_info['ilo_boot_iso']. Either %s "
+                                  "instance_info['boot_iso']. Either %s "
                                   "is not a valid HTTPS URL or is not "
                                   "reachable.", boot_iso)
             return
@@ -1021,12 +916,7 @@ class IloUefiHttpsBoot(base.BootInterface):
             parameters
         :raises: UnsupportedDriverExtension
         """
-        try:
-            self._validate_driver_info(task)
-        except exception.MissingParameterValue:
-            # Fall back to non-managed in-band inspection
-            raise exception.UnsupportedDriverExtension(
-                driver=task.node.driver, extension='inspection')
+        self._validate_driver_info(task)
 
     @METRICS.timer('IloUefiHttpsBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
@@ -1048,24 +938,18 @@ class IloUefiHttpsBoot(base.BootInterface):
         :raises: IloOperationError, if some operation on iLO failed.
         """
         node = task.node
-        # NOTE(TheJulia): If this method is being called by something
-        # aside from deployment, clean and rescue, such as conductor takeover,
-        # we should treat this as a no-op and move on otherwise we would
-        # modify the state of the node due to virtual media operations.
-        if node.provision_state not in (states.DEPLOYING,
-                                        states.CLEANING,
-                                        states.RESCUING,
-                                        states.INSPECTING):
+        if not driver_utils.need_prepare_ramdisk(node):
             return
 
         prepare_node_for_deploy(task)
 
-        # Clear ilo_boot_iso if it's a glance image to force recreate
+        # Clear boot_iso if it's a glance image to force recreate
         # another one again (or use existing one in glance).
         # This is mainly for rebuild and rescue scenario.
         if service_utils.is_glance_image(
                 node.instance_info.get('image_source')):
             instance_info = node.instance_info
+            instance_info.pop('boot_iso', None)
             instance_info.pop('ilo_boot_iso', None)
             node.instance_info = instance_info
             node.save()
@@ -1081,6 +965,9 @@ class IloUefiHttpsBoot(base.BootInterface):
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
 
+        # Signal to IPA that this is a vmedia boot operation.
+        ramdisk_params['boot_method'] = 'vmedia'
+
         mode = 'deploy'
         if node.provision_state == states.RESCUING:
             mode = 'rescue'
@@ -1090,6 +977,12 @@ class IloUefiHttpsBoot(base.BootInterface):
         iso_ref = image_utils.prepare_deploy_iso(task, ramdisk_params,
                                                  mode, d_info)
 
+        # NOTE(vmud213): Do not call if it is in the middle of
+        # clear_ca_certificates clean step as the TLS pending settings will
+        # be overridden
+        if not node.driver_internal_info.get('clear_ca_certs_flag'):
+            ilo_common.add_certificates(task)
+
         LOG.debug("Set 'UEFIHTTP' as one time boot option on the node "
                   "%(node)s to boot from URL %(iso_ref)s.",
                   {'node': node.uuid, 'iso_ref': iso_ref})
@@ -1142,6 +1035,8 @@ class IloUefiHttpsBoot(base.BootInterface):
             LOG.debug("Node %(node)s is set to permanently boot from local "
                       "%(device)s", {'node': task.node.uuid,
                                      'device': boot_devices.DISK})
+            # Need to enable secure boot, if being requested
+            boot_mode_utils.configure_secure_boot_if_needed(task)
             return
 
         params = {}
@@ -1154,6 +1049,8 @@ class IloUefiHttpsBoot(base.BootInterface):
                     "node %s. Booting instance from disk anyway.", node.uuid)
                 manager_utils.node_set_boot_device(task, boot_devices.DISK,
                                                    persistent=True)
+                # Need to enable secure boot, if being requested
+                boot_mode_utils.configure_secure_boot_if_needed(task)
 
                 return
             params.update(root_uuid=root_uuid)
@@ -1163,10 +1060,12 @@ class IloUefiHttpsBoot(base.BootInterface):
 
         if boot_option != 'ramdisk':
             i_info = node.instance_info
-            i_info['ilo_boot_iso'] = iso_ref
+            i_info['boot_iso'] = iso_ref
             node.instance_info = i_info
             node.save()
 
+        # Need to enable secure boot, if being requested
+        boot_mode_utils.configure_secure_boot_if_needed(task)
         ilo_common.setup_uefi_https(task, iso_ref, persistent=True)
 
         LOG.debug("Node %(node)s is set to boot from UEFIHTTP "
@@ -1186,6 +1085,7 @@ class IloUefiHttpsBoot(base.BootInterface):
                   "%(node)s", {'node': task.node.uuid})
 
         image_utils.cleanup_iso_image(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
     @METRICS.timer('IloUefiHttpsBoot.validate_rescue')
     def validate_rescue(self, task):
