@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
 from urllib.parse import urlparse
 
 from oslo_log import log
@@ -109,6 +110,14 @@ class RedfishFirmware(base.FirmwareInterface):
             LOG.warning('No manager available to retrieve Firmware '
                         'from the bmc of node %s', task.node.uuid)
 
+        nic_components = self.retrieve_nic_components(task, system)
+        if nic_components == []:
+            LOG.debug('Could not retrieve Firmware Package Version from '
+                      'NetworkAdapters on node %(node_uuid)s',
+                      {'node_uuid': task.node.uuid})
+        else:
+            settings.extend(nic_components)
+
         if not settings:
             error_msg = (_('Cannot retrieve firmware for node %s: no '
                            'supported components') % task.node.uuid)
@@ -139,6 +148,31 @@ class RedfishFirmware(base.FirmwareInterface):
                 up_fw_cmp.current_version = up_fw.get('current_version')
                 up_fw_cmp.save()
 
+    def retrieve_nic_components(self, task, system):
+        """Helper function to retrieve all NICs components on a given node.
+
+        :param task: a TaskManager instance.
+        :param system: a Redfish System object
+        :returns: a list of NIC components
+        """
+        nic_list = []
+        try:
+            chassis = redfish_utils.get_chassis(task.node, system)
+        except exception.RedfishError:
+            LOG.warning('No chassis available to retrieve NetworkAdapters '
+                        'firmware information on node %(node_uuid)s',
+                        {'node_uuid': task.node.uuid})
+            return nic_list
+
+        for net_adp in chassis.network_adapters.get_members():
+            for net_adp_ctrl in net_adp.controllers:
+                fw_pkg_v = net_adp_ctrl.firmware_package_version
+                net_adp_fw = {'component': redfish_utils.NIC_COMPONENT_PREFIX
+                              + net_adp.identity, 'current_version': fw_pkg_v}
+                nic_list.append(net_adp_fw)
+
+        return nic_list
+
     @METRICS.timer('RedfishFirmware.update')
     @base.deploy_step(priority=0, argsinfo=_FW_SETTINGS_ARGSINFO)
     @base.clean_step(priority=0, abortable=False,
@@ -146,7 +180,7 @@ class RedfishFirmware(base.FirmwareInterface):
                      requires_ramdisk=True)
     @base.service_step(priority=0, abortable=False,
                        argsinfo=_FW_SETTINGS_ARGSINFO,
-                       requires_ramdisk=True)
+                       requires_ramdisk=False)
     @base.cache_firmware_components
     def update(self, task, settings):
         """Update the Firmware on the node using the settings for components.
@@ -181,7 +215,8 @@ class RedfishFirmware(base.FirmwareInterface):
             polling=True
         )
 
-        return deploy_utils.reboot_to_finish_step(task, timeout=wait_interval)
+        return deploy_utils.reboot_to_finish_step(task, timeout=wait_interval,
+                                                  disable_ramdisk=True)
 
     def _execute_firmware_update(self, node, update_service, settings):
         """Executes the next firmware update to the node
@@ -232,6 +267,27 @@ class RedfishFirmware(base.FirmwareInterface):
                       {'node': node.uuid, 'error': e.message})
             raise exception.RedfishError(error=e)
 
+        # NOTE(iurygregory): In case we are doing firmware updates we need to
+        # account for unresponsive BMC, in this case we wait for a set of
+        # minutes before proceeding to the power actions.
+        # In case the node has firmware_update_unresponsive_bmc_wait set we
+        # give priority over the configuration option.
+        wait_unres_bmc = (
+            node.driver_info.get('firmware_update_unresponsive_bmc_wait')
+            or CONF.redfish.firmware_update_wait_unresponsive_bmc
+        )
+        LOG.debug('BMC firmware update in progress. Waiting %(wait_time)s '
+                  'seconds before proceeding to reboot the node %(node_uuid)s '
+                  'to complete the step', {'node_uuid': node.uuid,
+                                           'wait_time': wait_unres_bmc})
+        # TODO(iurygregory): Improve the logic here to identify if the BMC
+        # is back, so we don't have to unconditionally wait.
+        # The wait_unres_bmc will be the maximum time to wait.
+        time.sleep(wait_unres_bmc)
+        LOG.debug('Wait completed. Proceeding to reboot the node '
+                  '%(node_uuid)s to complete the step.',
+                  {'node_uuid': node.uuid})
+
         fw_upd['task_monitor'] = task_monitor.task_monitor_uri
         node.set_driver_internal_info('redfish_fw_updates', settings)
 
