@@ -28,8 +28,6 @@ import sys
 import tempfile
 import warnings
 
-import eventlet
-eventlet.monkey_patch(os=False)
 import fixtures
 from oslo_concurrency import processutils
 from oslo_config import fixture as config_fixture
@@ -58,9 +56,6 @@ logging.setup(CONF, 'ironic')
 BASE_TEST_TIMEOUT = os.environ.get('BASE_TEST_TIMEOUT', 60)
 
 
-BASE_TEST_TIMEOUT = os.environ.get('BASE_TEST_TIMEOUT', 60)
-
-
 class WarningsFixture(fixtures.Fixture):
     """Filters out warnings during test runs."""
 
@@ -119,15 +114,6 @@ class WarningsFixture(fixtures.Fixture):
             category=sqla_exc.SAWarning,
         )
 
-        # FIXME(stephenfin): We can remove this once oslo.db is fixed
-        # https://review.opendev.org/c/openstack/oslo.db/+/856453
-        warnings.filterwarnings(
-            'ignore',
-            module='ironic',
-            message='TypeDecorator .* will not produce a cache key',
-            category=sqla_exc.SAWarning,
-        )
-
         self.addCleanup(self._reset_warning_filters)
 
     def _reset_warning_filters(self):
@@ -216,6 +202,7 @@ class TestCase(oslo_test_base.BaseTestCase):
                     group='neutron')
         self.config(enabled_hardware_types=['fake-hardware',
                                             'manual-management'])
+        self.config(response_validation='error', group='api')
         self.config(initial_grub_template=None, group='pxe')
         for iface in drivers_base.ALL_INTERFACES:
             default = None
