@@ -44,9 +44,19 @@ class TestListDrivers(base.BaseApiTest):
         })
         for c in (c1, c2):
             self.dbapi.register_conductor_hardware_interfaces(
-                c.id, self.hw1, 'deploy', ['iscsi', 'direct'], 'direct')
+                c.id,
+                [{'hardware_type': self.hw1, 'interface_type': 'deploy',
+                  'interface_name': 'ansible', 'default': False},
+                 {'hardware_type': self.hw1, 'interface_type': 'deploy',
+                  'interface_name': 'direct', 'default': True}]
+            )
         self.dbapi.register_conductor_hardware_interfaces(
-            c1.id, self.hw2, 'deploy', ['iscsi', 'direct'], 'direct')
+            c1.id,
+            [{'hardware_type': self.hw2, 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': False},
+             {'hardware_type': self.hw2, 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': True}]
+        )
 
     def _test_drivers(self, use_dynamic, detail=False, latest_if=False):
         self.register_fake_conductors()
@@ -114,7 +124,7 @@ class TestListDrivers(base.BaseApiTest):
                 {
                     'hardware_type': self.hw1,
                     'interface_type': 'deploy',
-                    'interface_name': 'iscsi',
+                    'interface_name': 'ansible',
                     'default': False,
                 },
                 {
@@ -181,7 +191,8 @@ class TestListDrivers(base.BaseApiTest):
         self.assertThat(data['drivers'], matchers.HasLength(0))
         self.assertEqual([], data['drivers'])
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'get_driver_properties')
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_driver_properties',
+                       autospec=True)
     def _test_drivers_get_one_ok(self, mock_driver_properties,
                                  latest_if=False):
         # get_driver_properties mock is required by validate_link()
@@ -208,7 +219,7 @@ class TestListDrivers(base.BaseApiTest):
 
         for iface in driver_base.ALL_INTERFACES:
             if iface != 'bios':
-                if latest_if or iface not in ['rescue', 'storage']:
+                if latest_if or iface not in ['rescue', 'storage', 'firmware']:
                     self.assertIn('default_%s_interface' % iface, data)
                     self.assertIn('enabled_%s_interfaces' % iface, data)
 
@@ -227,7 +238,7 @@ class TestListDrivers(base.BaseApiTest):
                 {
                     'hardware_type': self.hw1,
                     'interface_type': 'deploy',
-                    'interface_name': 'iscsi',
+                    'interface_name': 'ansible',
                     'default': False,
                 },
                 {
@@ -263,6 +274,77 @@ class TestListDrivers(base.BaseApiTest):
         response = self.get_json('/drivers/nope', expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
 
+    def test_drivers_collection_custom_fields(self):
+        self.register_fake_conductors()
+        fields = "name,hosts"
+        data = self.get_json('/drivers?fields=%s' % fields,
+                             headers={api_base.Version.string: '1.77'})
+        for data_driver in data['drivers']:
+            self.assertCountEqual(['name', 'hosts', 'links'], data_driver)
+
+    def test_get_one_custom_fields(self):
+        self.register_fake_conductors()
+        driver = self.hw1
+        fields = "name,hosts"
+        data = self.get_json('/drivers/%s?fields=%s' % (driver, fields),
+                             headers={api_base.Version.string: '1.77'})
+        self.assertCountEqual(['name', 'hosts', 'links'], data)
+
+    def test_get_custom_fields_invalid_api_version(self):
+        self.register_fake_conductors()
+        driver = self.hw1
+        fields = "name,hosts"
+
+        response = self.get_json('/drivers?fields=%s' % fields,
+                                 headers={api_base.Version.string: '1.76'},
+                                 expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.NOT_ACCEPTABLE, response.status_int)
+
+        response = self.get_json('/drivers/%s?fields=%s' % (driver, fields),
+                                 headers={api_base.Version.string: '1.76'},
+                                 expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.NOT_ACCEPTABLE, response.status_int)
+
+    def test_drivers_collection_custom_fields_with_detail_true(self):
+        self.register_fake_conductors()
+        fields = "name,hosts"
+        response = self.get_json('/drivers?detail=true&fields=%s' % fields,
+                                 headers={api_base.Version.string: '1.77'},
+                                 expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
+    def test_drivers_collection_custom_fields_with_detail_false(self):
+        self.register_fake_conductors()
+        fields = "name,hosts"
+        data = self.get_json('/drivers?fields=%s&detail=false' % fields,
+                             headers={api_base.Version.string: '1.77'})
+        for data_driver in data['drivers']:
+            self.assertCountEqual(['name', 'hosts', 'links'], data_driver)
+
+    def test_drivers_collection_invalid_custom_fields(self):
+        self.register_fake_conductors()
+        fields = "name,invalid"
+        response = self.get_json('/drivers?fields=%s' % fields,
+                                 headers={api_base.Version.string: '1.77'},
+                                 expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertIn('invalid', response.json['error_message'])
+
+    def test_get_one_invalid_custom_fields(self):
+        self.register_fake_conductors()
+        driver = self.hw1
+        fields = "name,invalid"
+        response = self.get_json('/drivers/%s?fields=%s' % (driver, fields),
+                                 headers={api_base.Version.string: '1.77'},
+                                 expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertIn('invalid', response.json['error_message'])
+
     def _test_links(self, public_url=None):
         cfg.CONF.set_override('public_endpoint', public_url, 'api')
         self.register_fake_conductors()
@@ -289,7 +371,8 @@ class TestListDrivers(base.BaseApiTest):
     def test_links_public_url(self):
         self._test_links(public_url='http://foo')
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru')
+    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru',
+                       autospec=True)
     def test_driver_vendor_passthru_sync(self, mocked_driver_vendor_passthru):
         self.register_fake_conductors()
         mocked_driver_vendor_passthru.return_value = {
@@ -303,7 +386,8 @@ class TestListDrivers(base.BaseApiTest):
         self.assertEqual(mocked_driver_vendor_passthru.return_value['return'],
                          response.json)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru')
+    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru',
+                       autospec=True)
     def test_driver_vendor_passthru_async(self, mocked_driver_vendor_passthru):
         self.register_fake_conductors()
         mocked_driver_vendor_passthru.return_value = {'return': None,
@@ -315,7 +399,8 @@ class TestListDrivers(base.BaseApiTest):
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertIsNone(mocked_driver_vendor_passthru.return_value['return'])
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru')
+    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru',
+                       autospec=True)
     def test_driver_vendor_passthru_put(self, mocked_driver_vendor_passthru):
         self.register_fake_conductors()
         return_value = {'return': None, 'async': True, 'attach': False}
@@ -324,9 +409,10 @@ class TestListDrivers(base.BaseApiTest):
             '/drivers/%s/vendor_passthru/do_test' % self.hw1,
             {'test_key': 'test_value'})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
-        self.assertEqual(return_value['return'], response.json)
+        self.assertEqual(b'', response.body)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru')
+    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru',
+                       autospec=True)
     def test_driver_vendor_passthru_get(self, mocked_driver_vendor_passthru):
         self.register_fake_conductors()
         return_value = {'return': 'foo', 'async': False, 'attach': False}
@@ -335,7 +421,8 @@ class TestListDrivers(base.BaseApiTest):
             '/drivers/%s/vendor_passthru/do_test' % self.hw1)
         self.assertEqual(return_value['return'], response)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru')
+    @mock.patch.object(rpcapi.ConductorAPI, 'driver_vendor_passthru',
+                       autospec=True)
     def test_driver_vendor_passthru_delete(self, mock_driver_vendor_passthru):
         self.register_fake_conductors()
         return_value = {'return': None, 'async': True, 'attach': False}
@@ -343,7 +430,7 @@ class TestListDrivers(base.BaseApiTest):
         response = self.delete(
             '/drivers/%s/vendor_passthru/do_test' % self.hw1)
         self.assertEqual(http_client.ACCEPTED, response.status_int)
-        self.assertEqual(return_value['return'], response.json)
+        self.assertEqual(b'', response.body)
 
     def test_driver_vendor_passthru_driver_not_found(self):
         # tests when given driver is not found
@@ -363,11 +450,12 @@ class TestListDrivers(base.BaseApiTest):
 
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
         error = json.loads(response.json['error_message'])
-        self.assertEqual('Missing argument: "method"',
+        self.assertEqual('Missing mandatory parameter: method',
                          error['faultstring'])
 
     @mock.patch.object(rpcapi.ConductorAPI,
-                       'get_driver_vendor_passthru_methods')
+                       'get_driver_vendor_passthru_methods',
+                       autospec=True)
     def test_driver_vendor_passthru_methods(self, get_methods_mock):
         self.register_fake_conductors()
         return_value = {'foo': 'bar'}
@@ -376,7 +464,7 @@ class TestListDrivers(base.BaseApiTest):
 
         data = self.get_json(path)
         self.assertEqual(return_value, data)
-        get_methods_mock.assert_called_once_with(mock.ANY, self.hw1,
+        get_methods_mock.assert_called_once_with(mock.ANY, mock.ANY, self.hw1,
                                                  topic=mock.ANY)
 
         # Now let's test the cache: Reset the mock
@@ -388,7 +476,8 @@ class TestListDrivers(base.BaseApiTest):
         # Assert RPC method wasn't called this time
         self.assertFalse(get_methods_mock.called)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties')
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties',
+                       autospec=True)
     def test_raid_logical_disk_properties(self, disk_prop_mock):
         driver._RAID_PROPERTIES = {}
         self.register_fake_conductors()
@@ -398,10 +487,11 @@ class TestListDrivers(base.BaseApiTest):
         data = self.get_json(path,
                              headers={api_base.Version.string: "1.12"})
         self.assertEqual(properties, data)
-        disk_prop_mock.assert_called_once_with(mock.ANY, self.hw1,
+        disk_prop_mock.assert_called_once_with(mock.ANY, mock.ANY, self.hw1,
                                                topic=mock.ANY)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties')
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties',
+                       autospec=True)
     def test_raid_logical_disk_properties_older_version(self, disk_prop_mock):
         driver._RAID_PROPERTIES = {}
         self.register_fake_conductors()
@@ -413,7 +503,8 @@ class TestListDrivers(base.BaseApiTest):
                             expect_errors=True)
         self.assertEqual(http_client.NOT_ACCEPTABLE, ret.status_code)
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties')
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties',
+                       autospec=True)
     def test_raid_logical_disk_properties_cached(self, disk_prop_mock):
         # only one RPC-conductor call will be made and the info cached
         # for subsequent requests
@@ -426,11 +517,12 @@ class TestListDrivers(base.BaseApiTest):
             data = self.get_json(path,
                                  headers={api_base.Version.string: "1.12"})
             self.assertEqual(properties, data)
-        disk_prop_mock.assert_called_once_with(mock.ANY, self.hw1,
+        disk_prop_mock.assert_called_once_with(mock.ANY, mock.ANY, self.hw1,
                                                topic=mock.ANY)
         self.assertEqual(properties, driver._RAID_PROPERTIES[self.hw1])
 
-    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties')
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_raid_logical_disk_properties',
+                       autospec=True)
     def test_raid_logical_disk_properties_iface_not_supported(
             self, disk_prop_mock):
         driver._RAID_PROPERTIES = {}
@@ -443,12 +535,14 @@ class TestListDrivers(base.BaseApiTest):
                             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, ret.status_code)
         self.assertTrue(ret.json['error_message'])
-        disk_prop_mock.assert_called_once_with(mock.ANY, self.hw1,
+        disk_prop_mock.assert_called_once_with(mock.ANY, mock.ANY, self.hw1,
                                                topic=mock.ANY)
 
 
-@mock.patch.object(rpcapi.ConductorAPI, 'get_driver_properties')
-@mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for_driver')
+@mock.patch.object(rpcapi.ConductorAPI, 'get_driver_properties',
+                   autospec=True)
+@mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for_driver',
+                   autospec=True)
 class TestDriverProperties(base.BaseApiTest):
 
     def test_driver_properties_fake(self, mock_topic, mock_properties):
@@ -459,8 +553,9 @@ class TestDriverProperties(base.BaseApiTest):
         mock_properties.return_value = {'prop1': 'Property 1. Required.'}
         data = self.get_json('/drivers/%s/properties' % driver_name)
         self.assertEqual(mock_properties.return_value, data)
-        mock_topic.assert_called_once_with(driver_name)
-        mock_properties.assert_called_once_with(mock.ANY, driver_name,
+        mock_topic.assert_called_once_with(mock.ANY, driver_name)
+        mock_properties.assert_called_once_with(mock.ANY, mock.ANY,
+                                                driver_name,
                                                 topic=mock_topic.return_value)
         self.assertEqual(mock_properties.return_value,
                          driver._DRIVER_PROPERTIES[driver_name])
@@ -478,8 +573,9 @@ class TestDriverProperties(base.BaseApiTest):
             data = self.get_json('/drivers/%s/properties' % driver_name)
 
         self.assertEqual(mock_properties.return_value, data)
-        mock_topic.assert_called_once_with(driver_name)
-        mock_properties.assert_called_once_with(mock.ANY, driver_name,
+        mock_topic.assert_called_once_with(mock.ANY, driver_name)
+        mock_properties.assert_called_once_with(mock.ANY, mock.ANY,
+                                                driver_name,
                                                 topic=mock_topic.return_value)
         self.assertEqual(mock_properties.return_value,
                          driver._DRIVER_PROPERTIES[driver_name])
@@ -500,8 +596,9 @@ class TestDriverProperties(base.BaseApiTest):
             data = self.get_json('/drivers/%s/properties' % driver_name)
 
         self.assertEqual(mock_properties.return_value, data)
-        mock_topic.assert_called_once_with(driver_name)
-        mock_properties.assert_called_once_with(mock.ANY, driver_name,
+        mock_topic.assert_called_once_with(mock.ANY, driver_name)
+        mock_properties.assert_called_once_with(mock.ANY, mock.ANY,
+                                                driver_name,
                                                 topic=mock_topic.return_value)
         self.assertEqual(mock_properties.return_value,
                          driver._DRIVER_PROPERTIES[driver_name])
@@ -518,7 +615,7 @@ class TestDriverProperties(base.BaseApiTest):
         ret = self.get_json('/drivers/%s/properties' % driver_name,
                             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, ret.status_int)
-        mock_topic.assert_called_once_with(driver_name)
+        mock_topic.assert_called_once_with(mock.ANY, driver_name)
         self.assertFalse(mock_properties.called)
 
     def test_driver_properties_cannot_load(self, mock_topic, mock_properties):
@@ -532,6 +629,7 @@ class TestDriverProperties(base.BaseApiTest):
         ret = self.get_json('/drivers/%s/properties' % driver_name,
                             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, ret.status_int)
-        mock_topic.assert_called_once_with(driver_name)
-        mock_properties.assert_called_once_with(mock.ANY, driver_name,
+        mock_topic.assert_called_once_with(mock.ANY, driver_name)
+        mock_properties.assert_called_once_with(mock.ANY, mock.ANY,
+                                                driver_name,
                                                 topic=mock_topic.return_value)
