@@ -26,6 +26,7 @@ from oslo_utils import uuidutils
 from ironic.api.controllers import base as api_base
 from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import ramdisk
+from ironic.common import exception
 from ironic.common import states
 from ironic.conductor import rpcapi
 from ironic.drivers.modules import inspect_utils
@@ -65,6 +66,19 @@ class TestLookup(test_api_base.BaseApiTest):
 
     def _check_config(self, data):
         expected_config = {
+            'agent_containers': {
+                'allow_arbitrary_containers': CONF.agent_containers
+                .allow_arbitrary_containers,
+                'allowed_containers': CONF.agent_containers
+                .allowed_containers,
+                'container_steps_file': CONF.agent_containers
+                .container_steps_file,
+                'runner': CONF.agent_containers.runner,
+                'pull_options': CONF.agent_containers.pull_options,
+                'run_options': CONF.agent_containers.run_options,
+                'container_conf_file': CONF.agent_containers
+                .container_conf_file,
+            },
             'metrics': {
                 'backend': 'statsd',
                 'prepend_host': CONF.metrics.agent_prepend_host,
@@ -95,6 +109,20 @@ class TestLookup(test_api_base.BaseApiTest):
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
 
+    def test_invalid_input_addresses(self):
+        response = self.get_json(
+            '/lookup?addresses=meow1',
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
+    def test_invalid_input_node_uuid(self):
+        response = self.get_json(
+            '/lookup?node_uuid=meow1',
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
     def test_not_found(self):
         response = self.get_json(
             '/lookup?addresses=%s' % ','.join(self.addresses),
@@ -205,7 +233,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
     def test_old_api_version(self):
         response = self.post_json(
             '/heartbeat/%s' % uuidutils.generate_uuid(),
-            {'callback_url': 'url'},
+            {'callback_url': 'https://url'},
             headers={api_base.Version.string: str(api_v1.min_version())},
             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
@@ -213,7 +241,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
     def test_node_not_found(self):
         response = self.post_json(
             '/heartbeat/%s' % uuidutils.generate_uuid(),
-            {'callback_url': 'url'},
+            {'callback_url': 'https://url'},
             headers={api_base.Version.string: str(api_v1.max_version())},
             expect_errors=True)
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
@@ -223,29 +251,45 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'x'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None, 'x',
-                                               None, None, None,
+                                               node.uuid, 'https://url', None,
+                                               'x', None, None, None,
                                                topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
-    def test_ok_with_json(self, mock_heartbeat):
+    def test_ok_for_anaconda(self, mock_heartbeat):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
-            '/heartbeat/%s.json' % node.uuid,
-            {'callback_url': 'url',
-             'agent_token': 'maybe some magic'},
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': '',
+             'agent_token': 'x'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
-                                               'maybe some magic',
+                                               node.uuid, '', None,
+                                               'x', None, None, None,
+                                               topic='test-topic')
+
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_ok_with_json(self, mock_heartbeat):
+        headers = {api_base.Version.string: '1.90'}
+        node = obj_utils.create_test_node(self.context)
+        response = self.post_json(
+            '/heartbeat/%s.json' % node.uuid,
+            {'callback_url': 'https://url',
+             'agent_token': 'maybe some magic'},
+            headers=headers)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual(b'', response.body)
+        mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
+                                               node.uuid, 'https://url',
+                                               None, 'maybe some magic',
                                                None, None, None,
                                                topic='test-topic')
 
@@ -254,13 +298,13 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context, name='test.1')
         response = self.post_json(
             '/heartbeat/%s' % node.name,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'token'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'token', None, None, None,
                                                topic='test-topic')
 
@@ -269,14 +313,15 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_version': '1.4.1',
              'agent_token': 'meow'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', '1.4.1',
+                                               node.uuid, 'https://url',
+                                               '1.4.1',
                                                'meow',
                                                None, None, None,
                                                topic='test-topic')
@@ -286,20 +331,31 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_version': '1.4.1'},
             headers={api_base.Version.string: '1.35'},
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
 
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_heartbeat_rejects_file_url(self, mock_heartbeat):
+        node = obj_utils.create_test_node(
+            self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': 'file:///path/to/the/wizzard'},
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_heartbeat_rejects_different_callback_url(self, mock_heartbeat):
         node = obj_utils.create_test_node(
             self.context,
-            driver_internal_info={'agent_url': 'url'})
+            driver_internal_info={'agent_url': 'https://url'})
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url2'},
+            {'callback_url': 'https://url2'},
             headers={api_base.Version.string: str(api_v1.max_version())},
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
@@ -309,13 +365,13 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'http://url',
              'agent_token': 'abcdef1'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'http://url', None,
                                                'abcdef1', None, None, None,
                                                topic='test-topic')
 
@@ -324,14 +380,14 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcdef1'},
             headers={api_base.Version.string: str(api_v1.max_version())})
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'meow', 'abcdef1', None, None,
                                                topic='test-topic')
 
@@ -340,7 +396,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_status': 'start',
              'agent_status_message': 'woof',
@@ -349,7 +405,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(http_client.ACCEPTED, response.status_int)
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
-                                               node.uuid, 'url', None,
+                                               node.uuid, 'https://url', None,
                                                'meow', 'abcdef1', 'start',
                                                'woof', topic='test-topic')
 
@@ -358,7 +414,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_status': 'invalid_state',
              'agent_status_message': 'woof',
@@ -372,7 +428,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcd'},
             headers={api_base.Version.string: '1.67'},
@@ -384,7 +440,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         node = obj_utils.create_test_node(self.context)
         response = self.post_json(
             '/heartbeat/%s' % node.uuid,
-            {'callback_url': 'url',
+            {'callback_url': 'https://url',
              'agent_token': 'meow',
              'agent_verify_ca': 'abcd',
              'agent_status': 'wow',
@@ -399,6 +455,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
 class TestLookupScopedRBAC(TestLookup):
 
     """Test class to execute the Lookup tests with RBAC enforcement."""
+
     def setUp(self):
         super(TestLookupScopedRBAC, self).setUp()
 
@@ -413,6 +470,7 @@ class TestLookupScopedRBAC(TestLookup):
 class TestHeartbeatScopedRBAC(TestHeartbeat):
 
     """Test class to execute the Heartbeat tests with RBAC enforcement."""
+
     def setUp(self):
         super(TestHeartbeatScopedRBAC, self).setUp()
 
@@ -540,3 +598,82 @@ class TestContinueInspectionScopedRBAC(TestContinueInspection):
         cfg.CONF.set_override('enforce_new_defaults', True,
                               group='oslo_policy')
         cfg.CONF.set_override('auth_strategy', 'keystone')
+
+
+@mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for', autospec=True,
+                   return_value='test-topic')
+@mock.patch.object(rpcapi.ConductorAPI, 'create_node', autospec=True)
+@mock.patch.object(rpcapi.ConductorAPI, 'continue_inspection', autospec=True)
+@mock.patch.object(inspect_utils, 'lookup_node', autospec=True,
+                   side_effect=inspect_utils.AutoEnrollPossible)
+class TestContinueInspectionAutoDiscovery(test_api_base.BaseApiTest):
+
+    def setUp(self):
+        super().setUp()
+        CONF.set_override('enabled', True, group='auto_discovery')
+        CONF.set_override('driver', 'fake-hardware', group='auto_discovery')
+        self.addresses = ['11:22:33:44:55:66', '66:55:44:33:22:11']
+        self.bmcs = ['192.0.2.42']
+        self.inventory = {
+            'bmc_address': self.bmcs[0],
+            'interfaces': [
+                {'mac_address': mac, 'name': f'em{i}'}
+                for i, mac in enumerate(self.addresses)
+            ],
+        }
+        self.data = {
+            'inventory': self.inventory,
+            'test': 42,
+        }
+        self.node = obj_utils.get_test_node(self.context,
+                                            uuid=uuidutils.generate_uuid(),
+                                            provision_state='enroll')
+
+    def test_enroll(self, mock_lookup, mock_continue, mock_create,
+                    mock_get_topic):
+        mock_create.return_value = self.node
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual({'uuid': self.node.uuid}, response.json)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42, 'auto_discovered': True},
+            topic='test-topic')
+        new_node = mock_create.call_args.args[2]  # create(self, context, node)
+        self.assertEqual('fake-hardware', new_node.driver)
+        self.assertIsNone(new_node.resource_class)
+        self.assertEqual('', new_node.conductor_group)
+        self.assertEqual('enroll', new_node.provision_state)
+
+    def test_wrong_driver(self, mock_lookup, mock_continue, mock_create,
+                          mock_get_topic):
+        mock_get_topic.side_effect = exception.NoValidHost()
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            expect_errors=True)
+        self.assertEqual(http_client.INTERNAL_SERVER_ERROR,
+                         response.status_int)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_create.assert_not_called()
+        mock_continue.assert_not_called()
+
+    def test_override_defaults(self, mock_lookup, mock_continue, mock_create,
+                               mock_get_topic):
+        CONF.set_override('default_resource_class', 'xlarge-1')
+        # TODO(dtantsur): default_conductor_group
+        mock_create.return_value = self.node
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42, 'auto_discovered': True},
+            topic='test-topic')
+        new_node = mock_create.call_args.args[2]  # create(self, context, node)
+        self.assertEqual('fake-hardware', new_node.driver)
+        self.assertEqual('xlarge-1', new_node.resource_class)
+        self.assertEqual('', new_node.conductor_group)
