@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from http import client as http_client
+
 from ironic.api.controllers.v1 import versions
 from ironic.tests.unit.api import base
 
@@ -35,6 +37,19 @@ class TestRoot(base.BaseApiTest):
                          version1['min_version'])
         self.assertEqual(versions.max_version_string(), version1['version'])
 
+    def test_no_html_errors(self):
+        response = self.get_json('/foo', expect_errors=True)
+        self.assertEqual(http_client.NOT_FOUND, response.status_int)
+        self.assertIn('Not Found', response.json['error_message'])
+        self.assertNotIn('<html', response.json['error_message'])
+
+    def test_no_html_errors2(self):
+        response = self.delete('/', expect_errors=True)
+        self.assertEqual(http_client.METHOD_NOT_ALLOWED, response.status_int)
+        self.assertIn('malformed or otherwise incorrect',
+                      response.json['error_message'])
+        self.assertNotIn('<html', response.json['error_message'])
+
 
 class TestV1Root(base.BaseApiTest):
 
@@ -49,13 +64,20 @@ class TestV1Root(base.BaseApiTest):
         for f in data:
             self.assertNotIn(f, ['', []])
         # Check if all known resources are present and there are no extra ones.
-        not_resources = ('id', 'links', 'media_types')
+        not_resources = ('id', 'links', 'media_types', 'version')
         actual_resources = tuple(set(data) - set(not_resources))
         expected_resources = (['chassis', 'drivers', 'nodes', 'ports']
                               + additional_expected_resources)
         self.assertEqual(sorted(expected_resources), sorted(actual_resources))
-        self.assertIn({'type': 'application/vnd.openstack.ironic.v1+json',
-                       'base': 'application/json'}, data['media_types'])
+        self.assertEqual({'type': 'application/vnd.openstack.ironic.v1+json',
+                          'base': 'application/json'}, data['media_types'])
+
+        version1 = data['version']
+        self.assertEqual('v1', version1['id'])
+        self.assertEqual('CURRENT', version1['status'])
+        self.assertEqual(versions.min_version_string(),
+                         version1['min_version'])
+        self.assertEqual(versions.max_version_string(), version1['version'])
 
     def test_get_v1_root(self):
         self._test_get_root()
