@@ -21,8 +21,10 @@ from oslo_utils import uuidutils
 
 from ironic.common import exception
 from ironic.common import images
+from ironic.common import lessee_sources
 from ironic.common import states
 from ironic.common import swift
+from ironic.conductor import configdrive_utils as cd_utils
 from ironic.conductor import deployments
 from ironic.conductor import steps as conductor_steps
 from ironic.conductor import task_manager
@@ -125,9 +127,13 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self._test__do_node_deploy_driver_exception(RuntimeError('test'),
                                                     unexpected=True)
 
+    @mock.patch.object(cd_utils, 'check_and_fix_configdrive',
+                       autospec=True)
     @mock.patch.object(deployments, '_store_configdrive', autospec=True)
-    def _test__do_node_deploy_ok(self, mock_store, configdrive=None,
-                                 expected_configdrive=None, fast_track=False):
+    def _test__do_node_deploy_ok(self, mock_store, mock_c_cd,
+                                 configdrive=None,
+                                 expected_configdrive=None,
+                                 fast_track=False):
         if fast_track:
             self.config(fast_track=True, group='deploy')
         expected_configdrive = expected_configdrive or configdrive
@@ -142,6 +148,8 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                 driver_internal_info={'agent_url': 'url',
                                       'agent_secret_token': 'token'})
             task = task_manager.TaskManager(self.context, self.node.uuid)
+            if expected_configdrive:
+                mock_c_cd.return_value = configdrive
 
             deployments.do_node_deploy(task, self.service.conductor.id,
                                        configdrive=configdrive)
@@ -151,6 +159,8 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertIsNone(self.node.last_error)
             mock_deploy.assert_called_once_with(mock.ANY, mock.ANY)
             if configdrive:
+                mock_c_cd.assert_called_once_with(
+                    task, configdrive)
                 mock_store.assert_called_once_with(task.node,
                                                    expected_configdrive)
             else:
@@ -174,11 +184,13 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__do_node_deploy_fast_track(self):
         self._test__do_node_deploy_ok(fast_track=True)
 
+    @mock.patch.object(cd_utils, 'check_and_fix_configdrive',
+                       autospec=True)
     @mock.patch.object(swift, 'SwiftAPI', autospec=True)
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.prepare',
                 autospec=True)
     def test__do_node_deploy_configdrive_swift_error(self, mock_prepare,
-                                                     mock_swift):
+                                                     mock_swift, mock_c_cd):
         CONF.set_override('configdrive_use_object_store', True,
                           group='deploy')
         self._start_service()
@@ -186,7 +198,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                                           provision_state=states.DEPLOYING,
                                           target_provision_state=states.ACTIVE)
         task = task_manager.TaskManager(self.context, node.uuid)
-
+        mock_c_cd.return_value = 'fake config drive'
         mock_swift.side_effect = exception.SwiftOperationError('error')
         self.assertRaises(exception.SwiftOperationError,
                           deployments.do_node_deploy, task,
@@ -197,10 +209,14 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertEqual(states.ACTIVE, node.target_provision_state)
         self.assertIsNotNone(node.last_error)
         self.assertFalse(mock_prepare.called)
+        self.assertTrue(mock_c_cd.called)
 
+    @mock.patch.object(cd_utils, 'check_and_fix_configdrive',
+                       autospec=True)
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.prepare',
                 autospec=True)
-    def test__do_node_deploy_configdrive_db_error(self, mock_prepare):
+    def test__do_node_deploy_configdrive_db_error(self, mock_prepare,
+                                                  mock_c_cd):
         self._start_service()
         node = obj_utils.create_test_node(self.context, driver='fake-hardware',
                                           provision_state=states.DEPLOYING,
@@ -208,6 +224,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         task = task_manager.TaskManager(self.context, node.uuid)
         task.node.save()
         expected_instance_info = dict(node.instance_info)
+        mock_c_cd.return_value = 'fake config drive'
         with mock.patch.object(dbapi.IMPL, 'update_node',
                                autospec=True) as mock_db:
             db_node = self.dbapi.get_node_by_uuid(node.uuid)
@@ -237,12 +254,19 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             ]
             self.assertEqual(expected_calls, mock_db.mock_calls)
             self.assertFalse(mock_prepare.called)
+            self.assertTrue(mock_c_cd.called)
 
+    @mock.patch.object(cd_utils, 'check_and_fix_configdrive',
+                       autospec=True)
     @mock.patch.object(deployments, '_store_configdrive', autospec=True)
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.prepare',
                 autospec=True)
     def test__do_node_deploy_configdrive_unexpected_error(self, mock_prepare,
-                                                          mock_store):
+                                                          mock_store,
+                                                          mock_c_cd):
+        CONF.set_override('disable_configdrive_check', True,
+                          group="conductor")
+        mock_c_cd.return_value = 'fake config drive'
         self._start_service()
         node = obj_utils.create_test_node(self.context, driver='fake-hardware',
                                           provision_state=states.DEPLOYING,
@@ -259,6 +283,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertEqual(states.ACTIVE, node.target_provision_state)
         self.assertIsNotNone(node.last_error)
         self.assertFalse(mock_prepare.called)
+        self.assertFalse(mock_c_cd.called)
 
     def test__do_node_deploy_ok_2(self):
         # NOTE(rloo): a different way of testing for the same thing as in
@@ -282,6 +307,56 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertIsNone(node.last_error)
             mock_deploy.assert_called_once_with(mock.ANY, mock.ANY)
 
+    def test_node_validation_in_disabled_bios_boot_mode_fails(self):
+        self.config(disallowed_deployment_boot_modes=['bios'],
+                    group='conductor')
+        node = obj_utils.create_test_node(self.context,
+                                          properties={'boot_mode': 'bios'},
+                                          driver='fake-hardware')
+
+        with task_manager.acquire(self.context, node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.BootModeNotAllowed,
+                              deployments.validate_node,
+                              task, event='deploy')
+
+    def test_node_validation_in_disabled_uefi_boot_mode_fails(self):
+        self.config(disallowed_deployment_boot_modes=['uefi'],
+                    group='conductor')
+        node = obj_utils.create_test_node(self.context,
+                                          properties={'boot_mode': 'uefi'},
+                                          driver='fake-hardware')
+
+        with task_manager.acquire(self.context, node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.BootModeNotAllowed,
+                              deployments.validate_node,
+                              task, event='deploy')
+
+    def test_update_fails_on_invalid_boot_mode(self):
+        # NOTE(cid): This test might need updating if boot modes' naming
+        # convention changes
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['BIOS'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['Bios'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['UEFI'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['Uefi'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['blah'],
+                          group='conductor')
+
     @mock.patch.object(deployments, 'do_next_deploy_step', autospec=True)
     @mock.patch.object(conductor_steps, 'set_node_deployment_steps',
                        autospec=True)
@@ -341,19 +416,29 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                            mock_validate_deploy_user_steps_and_templates,
                            mock_deploy_validate,
                            mock_power_validate, mock_process_event,
-                           automatic_lessee=False):
+                           automatic_lessee=None, iinfo=None,
+                           automatic_lessee_source=None):
         self.context.auth_token_info = {
             'token': {'project': {'id': 'user-project'}}
         }
-        if automatic_lessee:
-            self.config(automatic_lessee=True, group='conductor')
+        if iinfo is None:
+            iinfo = {}
+
+        if automatic_lessee is not None:
+            self.config(automatic_lessee=automatic_lessee, group='conductor')
+
+        if automatic_lessee_source is not None:
+            self.config(automatic_lessee_source=automatic_lessee_source,
+                        group='conductor')
+
         self._start_service()
         mock_iwdi.return_value = False
         deploy_steps = [{"interface": "bios", "step": "factory_reset",
                          "priority": 95}]
         node = obj_utils.create_test_node(self.context, driver='fake-hardware',
                                           provision_state=states.AVAILABLE,
-                                          target_provision_state=states.ACTIVE)
+                                          target_provision_state=states.ACTIVE,
+                                          instance_info=iinfo)
         task = task_manager.TaskManager(self.context, node.uuid)
 
         deployments.start_deploy(task, self.service, configdrive=None,
@@ -372,18 +457,87 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             mock.ANY, 'deploy', call_args=(
                 deployments.do_node_deploy, task, 1, None, deploy_steps),
             callback=mock.ANY, err_handler=mock.ANY)
-        if automatic_lessee:
-            self.assertEqual('user-project', node.lessee)
+        expected_project = iinfo.get('project_id', 'user-project')
+        if (automatic_lessee_source not in [None, lessee_sources.NONE]
+                and automatic_lessee in [None, True]):
+            self.assertEqual(expected_project, node.lessee)
             self.assertIn('automatic_lessee', node.driver_internal_info)
         else:
             self.assertIsNone(node.lessee)
             self.assertNotIn('automatic_lessee', node.driver_internal_info)
 
-    def test_start_deploy(self):
+    def test_start_deploy_lessee_legacy_false(self):
         self._test_start_deploy(automatic_lessee=False)
 
-    def test_start_deploy_records_lessee(self):
-        self._test_start_deploy(automatic_lessee=True)
+    def test_start_deploy_lessee_source_none(self):
+        self._test_start_deploy(automatic_lessee_source=lessee_sources.NONE)
+
+    def test_start_deploy_lessee_source_request(self):
+        """Validates that project_id from request context is the lessee."""
+        self._test_start_deploy(automatic_lessee_source=lessee_sources.REQUEST)
+
+    def test_start_deploy_lessee_source_instance(self):
+        """Validates that project_id from instance info is the lessee."""
+        self._test_start_deploy(
+            automatic_lessee_source=lessee_sources.INSTANCE,
+            iinfo={'project_id': 'user-project-iinfo'})
+
+    @mock.patch.object(task_manager.TaskManager, 'process_event',
+                       autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakePower.validate',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakeDeploy.validate',
+                autospec=True)
+    @mock.patch.object(conductor_steps,
+                       'validate_user_deploy_steps_and_templates',
+                       autospec=True)
+    @mock.patch.object(conductor_utils, 'validate_instance_info_traits',
+                       autospec=True)
+    @mock.patch.object(images, 'is_whole_disk_image', autospec=True)
+    def test_start_deploy_lessee_legacy_false_even_if_src_set(
+            self, mock_iwdi, mock_validate_traits,
+            mock_validate_deploy_user_steps_and_templates,
+            mock_deploy_validate, mock_power_validate, mock_process_event):
+        self.context.auth_token_info = {
+            'token': {'project': {'id': 'user-project'}}
+        }
+        iinfo = {'project_id': 'user-project-iinfo'}
+
+        # Legacy lessee is disabled
+        self.config(automatic_lessee=False, group='conductor')
+        # but source is also set -- we should respect the disablement above all
+        self.config(automatic_lessee_source=lessee_sources.INSTANCE,
+                    group='conductor')
+
+        self._start_service()
+        mock_iwdi.return_value = False
+        deploy_steps = [{"interface": "bios", "step": "factory_reset",
+                         "priority": 95}]
+        node = obj_utils.create_test_node(self.context, driver='fake-hardware',
+                                          provision_state=states.AVAILABLE,
+                                          target_provision_state=states.ACTIVE,
+                                          instance_info=iinfo)
+        task = task_manager.TaskManager(self.context, node.uuid)
+
+        deployments.start_deploy(task, self.service, configdrive=None,
+                                 event='deploy', deploy_steps=deploy_steps)
+        node.refresh()
+        mock_iwdi.assert_called_once_with(task.context,
+                                          task.node.instance_info)
+        self.assertFalse(node.driver_internal_info['is_whole_disk_image'])
+        self.assertEqual('partition', node.instance_info['image_type'])
+        mock_power_validate.assert_called_once_with(task.driver.power, task)
+        mock_deploy_validate.assert_called_once_with(task.driver.deploy, task)
+        mock_validate_traits.assert_called_once_with(task.node)
+        mock_validate_deploy_user_steps_and_templates.assert_called_once_with(
+            task, deploy_steps, skip_missing=True)
+        mock_process_event.assert_called_with(
+            mock.ANY, 'deploy', call_args=(
+                deployments.do_node_deploy, task, 1, None, deploy_steps),
+            callback=mock.ANY, err_handler=mock.ANY)
+
+        self.assertIsNone(node.lessee)
+        self.assertNotIn('automatic_lessee', node.driver_internal_info)
 
     @mock.patch.object(images, 'is_source_a_path', autospec=True)
     @mock.patch.object(task_manager.TaskManager, 'process_event',
@@ -498,7 +652,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
             'step': 'deploy_middle', 'priority': 30, 'interface': 'deploy'}
 
     def test__do_next_deploy_step_none(self):
-        self._start_service()
         node = obj_utils.create_test_node(self.context, driver='fake-hardware')
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
@@ -513,7 +666,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
     def test__do_next_deploy_step_async(self, mock_execute):
         driver_internal_info = {'deploy_step_index': None,
                                 'deploy_steps': self.deploy_steps}
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             driver_internal_info=driver_internal_info,
@@ -537,8 +689,8 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
                 autospec=True)
     def test__do_next_deploy_step_in_deploywait(self, mock_execute):
         driver_internal_info = {'deploy_step_index': None,
-                                'deploy_steps': self.deploy_steps}
-        self._start_service()
+                                'deploy_steps': self.deploy_steps,
+                                'deployment_polling': True}
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             driver_internal_info=driver_internal_info,
@@ -562,6 +714,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertEqual(states.ACTIVE, node.target_provision_state)
         self.assertEqual(expected_first_step, node.deploy_step)
         self.assertEqual(0, node.driver_internal_info['deploy_step_index'])
+        self.assertNotIn('deployment_polling', node.driver_internal_info)
         mock_execute.assert_called_once_with(mock.ANY, task,
                                              self.deploy_steps[0])
 
@@ -571,7 +724,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # Resume an in-progress deploy after the first async step
         driver_internal_info = {'deploy_step_index': 0,
                                 'deploy_steps': self.deploy_steps}
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             provision_state=states.DEPLOYWAIT,
@@ -604,7 +756,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # Resume where last_step is the last deploy step that was executed
         driver_internal_info = {'deploy_step_index': 1,
                                 'deploy_steps': self.deploy_steps}
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             provision_state=states.DEPLOYWAIT,
@@ -684,7 +835,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
                                 'deploy_steps': self.deploy_steps[:1],
                                 'agent_url': 'url',
                                 'agent_secret_token': 'token'}
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             driver_internal_info=driver_internal_info,
@@ -760,7 +910,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # When a deploy step fails, go to DEPLOYFAIL
         driver_internal_info = {'deploy_step_index': None,
                                 'deploy_steps': self.deploy_steps}
-        self._start_service()
 
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
@@ -867,7 +1016,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # When a deploy step fails, go to DEPLOYWAIT
         tgt_prov_state = states.ACTIVE
 
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             provision_state=states.DEPLOYING,
@@ -884,7 +1032,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
                 self.context, node.uuid, shared=False) as task:
             deployments.do_next_deploy_step(task, 0)
 
-        self._stop_service()
         node.refresh()
 
         # Make sure we go to DEPLOYWAIT
@@ -902,7 +1049,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # When a deploy step fails, go to DEPLOYWAIT
         tgt_prov_state = states.ACTIVE
 
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             provision_state=states.DEPLOYING,
@@ -919,7 +1065,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
                 self.context, node.uuid, shared=False) as task:
             deployments.do_next_deploy_step(task, 0)
 
-        self._stop_service()
         node.refresh()
 
         # Make sure we go to DEPLOYWAIT
@@ -937,7 +1082,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         # When a deploy step fails with no reboot requested go to DEPLOYFAIL
         tgt_prov_state = states.ACTIVE
 
-        self._start_service()
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             provision_state=states.DEPLOYING,
@@ -953,7 +1097,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
                 self.context, node.uuid, shared=False) as task:
             deployments.do_next_deploy_step(task, 0)
 
-        self._stop_service()
         node.refresh()
 
         # Make sure we go to DEPLOYFAIL, clear deploy_steps
@@ -966,6 +1109,34 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         mock_execute.assert_called_once_with(
             mock.ANY, mock.ANY, self.deploy_steps[0])
 
+    def _test_do_next_deploy_step_handles(self, start_state):
+        node = obj_utils.create_test_node(
+            self.context, driver='fake-hardware',
+            provision_state=start_state,
+            driver_internal_info={
+                'deploy_steps': [
+                    {
+                        'step': 'hold',
+                        'priority': 10,
+                        'interface': 'deploy'
+                    }
+                ],
+                'deploy_step_index': None},
+            clean_step=None)
+        with task_manager.acquire(
+                self.context, node.uuid, shared=False) as task:
+            deployments.do_next_deploy_step(task, 0)
+        node.refresh()
+        self.assertEqual(states.DEPLOYHOLD, node.provision_state)
+
+    def test_do_next_deploy_step_handles_hold_from_active(self):
+        # Prior step/action was out of the conductor
+        self._test_do_next_deploy_step_handles(states.DEPLOYING)
+
+    def test_do_next_deploy_step_handles_hold_from_wait(self):
+        # Prior step was async in a wait state
+        self._test_do_next_deploy_step_handles(states.DEPLOYWAIT)
+
     @mock.patch.object(deployments, 'do_next_deploy_step', autospec=True)
     @mock.patch.object(conductor_steps, '_get_steps', autospec=True)
     def _continue_node_deploy(self, mock__get_steps, mock_next_step,
