@@ -13,8 +13,10 @@
 
 import datetime
 import json
+import time
 from unittest import mock
 
+from oslo_config import cfg
 from oslo_utils import timeutils
 import sushy
 
@@ -34,8 +36,6 @@ from ironic.tests.unit.objects import utils as obj_utils
 INFO_DICT = db_utils.get_test_redfish_info()
 
 
-@mock.patch('oslo_utils.eventletutils.EventletEvent.wait',
-            lambda *args, **kwargs: None)
 class RedfishFirmwareTestCase(db_base.DbTestCase):
 
     def setUp(self):
@@ -66,10 +66,11 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
     @mock.patch.object(redfish_fw, 'LOG', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     @mock.patch.object(redfish_utils, 'get_manager', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_chassis', autospec=True)
     @mock.patch.object(objects.FirmwareComponentList,
                        'sync_firmware_components', autospec=True)
-    def test_missing_all_components(self, sync_fw_cmp_mock, manager_mock,
-                                    system_mock, log_mock):
+    def test_missing_all_components(self, sync_fw_cmp_mock, chassis_mock,
+                                    manager_mock, system_mock, log_mock):
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
@@ -78,6 +79,10 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
             system_mock.return_value.bios_version = None
             manager_mock.return_value.firmware_version = None
 
+            netadp = mock.MagicMock()
+            netadp.get_members.return_value = []
+            chassis_mock.return_value.network_adapters = netadp
+
             self.assertRaises(exception.UnsupportedDriverExtension,
                               task.driver.firmware.cache_firmware_components,
                               task)
@@ -89,7 +94,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
                 % self.node.uuid)
             log_mock.error.assert_called_once_with(error_msg)
 
-            warning_calls = [
+            debug_calls = [
                 mock.call('Could not retrieve BiosVersion in node '
                           '%(node_uuid)s system %(system)s',
                           {'node_uuid': self.node.uuid,
@@ -97,8 +102,11 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
                 mock.call('Could not retrieve FirmwareVersion in node '
                           '%(node_uuid)s manager %(manager)s',
                           {'node_uuid': self.node.uuid,
-                           'manager': "Manager1"})]
-            log_mock.debug.assert_has_calls(warning_calls)
+                           'manager': "Manager1"}),
+                mock.call('Could not retrieve Firmware Package Version '
+                          'from NetworkAdapters on node %(node_uuid)s',
+                          {'node_uuid': self.node.uuid})]
+            log_mock.debug.assert_has_calls(debug_calls)
 
     @mock.patch.object(redfish_fw, 'LOG', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
@@ -126,7 +134,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
             task.driver.firmware.cache_firmware_components(task)
             system_mock.assert_called_once_with(task.node)
 
-            log_mock.debug.assert_called_once_with(
+            log_mock.debug.assert_any_call(
                 'Could not retrieve BiosVersion in node '
                 '%(node_uuid)s system %(system)s',
                 {'node_uuid': self.node.uuid, 'system': 'System1'})
@@ -160,7 +168,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
             system_mock.return_value.bios_version = "v1.0.0"
             task.driver.firmware.cache_firmware_components(task)
 
-            log_mock.debug.assert_called_once_with(
+            log_mock.debug.assert_any_call(
                 'Could not retrieve FirmwareVersion in node '
                 '%(node_uuid)s manager %(manager)s',
                 {'node_uuid': self.node.uuid, 'manager': "Manager1"})
@@ -174,13 +182,16 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
     @mock.patch.object(redfish_fw, 'LOG', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     @mock.patch.object(redfish_utils, 'get_manager', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_chassis', autospec=True)
     @mock.patch.object(objects, 'FirmwareComponentList', autospec=True)
     @mock.patch.object(objects, 'FirmwareComponent', spec_set=True,
                        autospec=True)
     def test_create_all_components(self, fw_cmp_mock, fw_cmp_list_mock,
-                                   manager_mock, system_mock, log_mock):
+                                   chassis_mock, manager_mock, system_mock,
+                                   log_mock):
         create_list = [{'component': 'bios', 'current_version': 'v1.0.0'},
-                       {'component': 'bmc', 'current_version': 'v1.0.0'}]
+                       {'component': 'bmc', 'current_version': 'v1.0.0'},
+                       {'component': 'nic:NIC1', 'current_version': '1'}]
         fw_cmp_list_mock.sync_firmware_components.return_value = (
             create_list, [], []
         )
@@ -191,10 +202,22 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
         bmc_component = {'component': 'bmc', 'current_version': 'v1.0.0',
                          'node_id': self.node.id}
 
+        nic_component = {'component': 'nic:NIC1', 'current_version': '1',
+                         'node_id': self.node.id}
+
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             manager_mock.return_value.firmware_version = "v1.0.0"
             system_mock.return_value.bios_version = "v1.0.0"
+            chassis_mock
+            netadp_ctrl = mock.MagicMock()
+            netadp_ctrl.firmware_package_version = "1"
+            netadp = mock.MagicMock()
+            netadp.identity = 'NIC1'
+            netadp.controllers = [netadp_ctrl]
+            net_adapters = mock.MagicMock()
+            net_adapters.get_members.return_value = [netadp]
+            chassis_mock.return_value.network_adapters = net_adapters
             task.driver.firmware.cache_firmware_components(task)
 
             log_mock.warning.assert_not_called()
@@ -203,15 +226,47 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
             fw_cmp_list_mock.sync_firmware_components.assert_called_once_with(
                 task.context, task.node.id,
                 [{'component': 'bios', 'current_version': 'v1.0.0'},
-                 {'component': 'bmc', 'current_version': 'v1.0.0'}])
+                 {'component': 'bmc', 'current_version': 'v1.0.0'},
+                 {'component': 'nic:NIC1', 'current_version': '1'}])
             fw_cmp_calls = [
                 mock.call(task.context, **bios_component),
                 mock.call().create(),
                 mock.call(task.context, **bmc_component),
                 mock.call().create(),
+                mock.call(task.context, **nic_component),
+                mock.call().create()
             ]
             fw_cmp_mock.assert_has_calls(fw_cmp_calls)
 
+    @mock.patch.object(redfish_fw, 'LOG', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_chassis', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_manager', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    @mock.patch.object(objects, 'FirmwareComponentList', autospec=True)
+    def test_get_chassis_redfish_error(self, sync_fw_cmp_mock, system_mock,
+                                       manager_mock, chassis_mock, log_mock):
+        system_mock.return_value.identity = "System1"
+        system_mock.return_value.bios_version = '1.0.0'
+        manager_mock.return_value.identity = "Manager1"
+        manager_mock.return_value.firmware_version = '1.0.0'
+
+        chassis_mock.side_effect = exception.RedfishError('not found')
+
+        sync_fw_cmp_mock.sync_firmware_components.return_value = (
+            [{'component': 'bios', 'current_version': '1.0.0'},
+             {'component': 'bmc', 'current_version': '1.0.0'},],
+            [], [])
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.driver.firmware.cache_firmware_components(task)
+
+        log_mock.warning.assert_any_call(
+            'No chassis available to retrieve NetworkAdapters firmware '
+            'information on node %(node_uuid)s',
+            {'node_uuid': self.node.uuid}
+        )
+
     @mock.patch.object(redfish_utils, 'LOG', autospec=True)
     @mock.patch.object(redfish_utils, '_get_connection', autospec=True)
     def test_missing_updateservice(self, conn_mock, log_mock):
@@ -298,7 +353,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
 
     def test_invalid_component_in_settings(self):
         argsinfo = {'settings': [
-            {'component': 'nic', 'url': 'https://nic-update/v1.1.0'}
+            {'component': 'something', 'url': 'https://nic-update/v1.1.0'}
         ]}
         self.node.clean_step = {'priority': 100, 'interface': 'firmware',
                                 'step': 'update',
@@ -308,7 +363,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
 
     def test_invalid_component_in_settings_service(self):
         argsinfo = {'settings': [
-            {'component': 'nic', 'url': 'https://nic-update/v1.1.0'}
+            {'component': 'something', 'url': 'https://nic-update/v1.1.0'}
         ]}
         self.node.service_step = {'priority': 100, 'interface': 'firmware',
                                   'step': 'update',
@@ -354,6 +409,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_invalid_settings_service()
 
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
     def _generate_new_driver_internal_info(self, components=[], invalid=False,
                                            add_wait=False, wait=1):
         bmc_component = {'component': 'bmc', 'url': 'https://bmc/v1.0.1'}
@@ -797,6 +853,8 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
     def test_continue_updates_more_updates(self, get_system_collection_mock,
                                            node_power_action_mock,
                                            log_mock):
+        cfg.CONF.set_override('firmware_update_wait_unresponsive_bmc', 0,
+                              'redfish')
         self._generate_new_driver_internal_info(['bmc', 'bios'])
 
         task_monitor_mock = mock.Mock()
@@ -831,6 +889,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
 
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system_collection', autospec=True)
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
     def test__execute_firmware_update_no_targets(self,
                                                  get_system_collection_mock,
                                                  system_mock):
@@ -857,6 +916,7 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
 
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system_collection', autospec=True)
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
     def test__execute_firmware_update_targets(self,
                                               get_system_collection_mock,
                                               system_mock):
@@ -880,3 +940,79 @@ class RedfishFirmwareTestCase(db_base.DbTestCase):
                                           settings)
         update_service_mock.simple_update.assert_called_once_with(
             'https://bios/v1.0.1', targets=[mock.ANY])
+
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system_collection', autospec=True)
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test__execute_firmware_update_unresponsive_bmc(self, sleep_mock,
+                                                       get_sys_collec_mock,
+                                                       system_mock):
+        cfg.CONF.set_override('firmware_update_wait_unresponsive_bmc', 1,
+                              'redfish')
+        self._generate_new_driver_internal_info(['bmc'])
+        with open(
+            'ironic/tests/json_samples/systems_collection_single.json'
+        ) as f:
+            resp_obj = json.load(f)
+        system_collection_mock = mock.MagicMock()
+        system_collection_mock.get_members.return_value = resp_obj['Members']
+        get_sys_collec_mock.return_value = system_collection_mock
+
+        task_monitor_mock = mock.Mock()
+        task_monitor_mock.task_monitor_uri = '/task/2'
+        update_service_mock = mock.Mock()
+        update_service_mock.simple_update.return_value = task_monitor_mock
+
+        firmware = redfish_fw.RedfishFirmware()
+
+        settings = [{'component': 'bmc', 'url': 'https://bmc/v1.2.3'}]
+        firmware._execute_firmware_update(self.node, update_service_mock,
+                                          settings)
+
+        update_service_mock.simple_update.assert_called_once_with(
+            'https://bmc/v1.2.3')
+        sleep_mock.assert_called_once_with(
+            CONF.redfish.firmware_update_wait_unresponsive_bmc)
+
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system_collection', autospec=True)
+    @mock.patch.object(time, 'sleep', autospec=True)
+    def test__execute_firmware_update_unresponsive_bmc_node_override(
+            self, sleep_mock, get_sys_collec_mock, system_mock):
+        self._generate_new_driver_internal_info(['bmc'])
+        # Set a specific value for firmware_update_unresponsive_bmc_wait for
+        # the node
+        with mock.patch('time.sleep', lambda x: None):
+            d_info = self.node.driver_info.copy()
+            d_info['firmware_update_unresponsive_bmc_wait'] = 1
+            self.node.driver_info = d_info
+            self.node.save()
+
+        self.assertNotEqual(
+            CONF.redfish.firmware_update_wait_unresponsive_bmc,
+            self.node.driver_info.get('firmware_update_unresponsive_bmc_wait')
+        )
+
+        with open(
+            'ironic/tests/json_samples/systems_collection_single.json'
+        ) as f:
+            resp_obj = json.load(f)
+        system_collection_mock = mock.MagicMock()
+        system_collection_mock.get_members.return_value = resp_obj['Members']
+        get_sys_collec_mock.return_value = system_collection_mock
+
+        task_monitor_mock = mock.Mock()
+        task_monitor_mock.task_monitor_uri = '/task/2'
+        update_service_mock = mock.Mock()
+        update_service_mock.simple_update.return_value = task_monitor_mock
+
+        firmware = redfish_fw.RedfishFirmware()
+        settings = [{'component': 'bmc', 'url': 'https://bmc/v1.2.3'}]
+        firmware._execute_firmware_update(self.node, update_service_mock,
+                                          settings)
+
+        update_service_mock.simple_update.assert_called_once_with(
+            'https://bmc/v1.2.3')
+        sleep_mock.assert_called_once_with(
+            self.node.driver_info.get('firmware_update_unresponsive_bmc_wait')
+        )
