@@ -20,8 +20,8 @@ import time
 from unittest import mock
 
 from oslo_config import cfg
-from oslo_utils import importutils
 import requests
+import sushy
 
 from ironic.common import exception
 from ironic.drivers.modules.redfish import utils as redfish_utils
@@ -29,8 +29,6 @@ from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
 
-sushy = importutils.try_import('sushy')
-
 INFO_DICT = db_utils.get_test_redfish_info()
 
 
@@ -61,6 +59,14 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         response = redfish_utils.parse_driver_info(self.node)
         self.assertEqual(self.parsed_driver_info, response)
 
+    def test_parse_driver_info_firmware_update_unresponsive_bmc_wait_set(
+            self):
+        self.node.driver_info['firmware_update_unresponsive_bmc_wait'] = 30
+        self.parsed_driver_info = redfish_utils.parse_driver_info(self.node)
+        self.assertEqual(
+            self.parsed_driver_info['firmware_update_unresponsive_bmc_wait'],
+            30)
+
     def test_parse_driver_info_default_scheme(self):
         self.node.driver_info['redfish_address'] = 'example.com'
         response = redfish_utils.parse_driver_info(self.node)
@@ -168,6 +174,14 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         response = redfish_utils.parse_driver_info(self.node)
         self.assertEqual(self.parsed_driver_info, response)
 
+    def test_parse_driver_info_default_scheme_ipv6_brackets_added(self):
+        test_redfish_address = '2001:DB8::1'
+        self.node.driver_info['redfish_address'] = test_redfish_address
+        response = redfish_utils.parse_driver_info(self.node)
+        self.parsed_driver_info['address'] = ("https://[%s]"
+                                              % test_redfish_address)
+        self.assertEqual(self.parsed_driver_info, response)
+
     def test_get_task_monitor(self):
         redfish_utils._get_connection = mock.Mock()
         fake_monitor = mock.Mock()
@@ -221,6 +235,25 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         self.assertRaises(exception.RedfishError,
                           redfish_utils.get_event_service, self.node)
 
+    def test_get_system_collection(self):
+        redfish_utils._get_connection = mock.Mock()
+        mock_system_collection = mock.Mock()
+        redfish_utils._get_connection.return_value = mock_system_collection
+
+        result = redfish_utils.get_system_collection(self.node)
+
+        self.assertEqual(mock_system_collection, result)
+
+    def test_get_system_collection_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.ResourceNotFoundError('GET',
+                                                   '/',
+                                                   requests.Response())
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_system_collection, self.node)
+
 
 class RedfishUtilsAuthTestCase(db_base.DbTestCase):
 
@@ -242,6 +275,7 @@ class RedfishUtilsAuthTestCase(db_base.DbTestCase):
             'password': 'password',
             'verify_ca': True,
             'auth_type': 'auto',
+            'firmware_update_unresponsive_bmc_wait': 300,
             'node_uuid': self.node.uuid
         }
 
@@ -396,6 +430,7 @@ class RedfishUtilsSystemTestCase(db_base.DbTestCase):
             'password': 'password',
             'verify_ca': True,
             'auth_type': 'auto',
+            'firmware_update_unresponsive_bmc_wait': 300,
             'node_uuid': self.node.uuid
         }
 
